# *** A way to run these unit tests. ***
# 1. (Open a terminal, alternately known as go to the command line.)
# 2. run "cd .." (Or, somehow make the project root the current directory.)
# 3. python -m unittest tests.ebay_rest

# Don't repeat API calls with the same parameters, because it appears to trigger an "Internal (Server) Error" at eBay.

# Whenever possible, run tests on the sandbox instead of production.

# Standard library imports
import datetime
from copy import deepcopy
from json import loads
import os
import random
import string
from typing import Optional
import unittest
from urllib.parse import urlparse

# 3rd party libraries
from currency_converter import CurrencyConverter

# Local imports
from src.ebay_rest import API, DateTime, Error, Reference


class APIBothEnvironmentsSingleSiteTests(unittest.TestCase):
    def test_start(
        self,
    ):
        try:
            # all initialization options and for each the first and second usage
            for environment in ("production", "sandbox"):
                for throttle, timeout in ((False, None), (True, None), (True, 60.0)):

                    if timeout is None:
                        api = API(
                            application=environment + "_1",
                            user=environment + "_1",
                            header="US",
                            throttle=throttle,
                        )
                    else:
                        api = API(
                            application=environment + "_1",
                            user=environment + "_1",
                            header="US",
                            throttle=throttle,
                            timeout=timeout,
                        )
                    item_id = None
                    for record in api.buy_browse_search(limit=1, q="lego"):
                        if "record" not in record:
                            self.assertTrue(
                                "total" in record, f"Unexpected non-record{record}."
                            )
                        else:
                            item = record["record"]
                            self.assertTrue(isinstance(item["item_id"], str))
                            item_id = item["item_id"]
                            break
                    if item_id:
                        item = api.buy_browse_get_item(item_id=item_id)
                        self.assertTrue(isinstance(item, dict))
        except Error as error:
            self.fail(f"Error {error.number} is {error.reason}  {error.detail}.\n")


class APIMarketplaceSpecificTests(unittest.TestCase):

    def test_belgium_nederlandse(self):
        api = API(application="production_1", user="production_1", header="NLBE")
        for record in api.buy_browse_search(limit=1, q="black"):
            if "record" not in record:
                self.assertGreater(
                    record["total"]["records_yielded"],
                    0,
                    "No item from Belgium found; try a different q value.",
                )
            else:
                item = record["record"]
                host = urlparse(item["item_web_url"]).hostname
                self.assertEqual(host, "www.benl.ebay.be")


class APISandboxMultipleSiteTests(unittest.TestCase):
    """API tests that require multiple marketplaces and that can be done on the sandbox."""

    @classmethod
    def setUpClass(cls):
        cls.currency_converter = CurrencyConverter()  # this is slow, so do it once

    def load_credential_file(self, filename: str) -> Optional[dict]:
        """Try to load a credential file into a dict."""
        config_location = os.path.join(os.getcwd(), filename)
        try:
            f = open(config_location, "r")
        except FileNotFoundError:
            self.fail(f"File not found, unable to open {config_location}.")
        else:
            try:
                contents = loads(f.read())
            except IOError:
                f.close()
                self.fail(
                    f"Unable to load the file {config_location}; likely the json is malformed."
                )
            else:
                f.close()
                return contents

    def check_credential_file(self, filename):
        """Check if a credential file is structured correctly."""

        # These are all the root keys.
        instructions_key = "***_instructions_***"
        applications_key = "applications"
        users_key = "users"
        headers_key = "headers"
        key_pairs_key = "key_pairs"
        root_keys = {
            instructions_key,
            applications_key,
            users_key,
            headers_key,
            key_pairs_key,
        }

        # Can the JSON file be loaded into a dictionary?
        contents = self.load_credential_file(filename)
        self.assertIsInstance(
            contents,
            dict,
            "Failed to load {filename} from the test folder into a dict.",
        )

        # Are the expected root keys present?
        root_keys_in_contents = list(contents.keys())
        self.assertEqual(len(root_keys_in_contents), len(root_keys))
        self.assertEqual(set(root_keys_in_contents), root_keys)

        # Check the common requirements for all root_keys records.
        for root_key in root_keys:
            keys = contents[root_key].keys()
            self.assertGreater(len(keys), 0, f"{filename} has no {root_key} records.")
            for key in keys:
                self.assertIsInstance(
                    key,
                    str,
                    f"In {filename} the {root_key} record named {key} is not a string.",
                )

        for instruction_key in contents[instructions_key]:
            try:
                int(instruction_key)
            except ValueError:
                self.fail(
                    f"In {filename} the instruction key {instruction_key} is not string representing an integer."
                )
            self.assertIsInstance(
                contents[instructions_key][instruction_key],
                str,
                f"In {filename} the value for instruction key {instruction_key} is not a string.",
            )

        for application in contents[applications_key]:
            pass  # TODO

        for user in contents[users_key]:
            pass  # TODO

        for header in contents[headers_key]:
            pass  # TODO

        for key_pair in contents[key_pairs_key]:
            pass  # TODO

    def test_credential_files(self):
        self.check_credential_file("ebay_rest_EXAMPLE.json")
        self.check_credential_file("ebay_rest.json")

    def test_credentials_from_dicts(self):
        """set credentials via dicts"""

        contents = self.load_credential_file("ebay_rest.json")
        try:
            application = contents["applications"]["sandbox_1"]
            user = contents["users"]["sandbox_1"]
            header = contents["headers"]["US"]
            key_pair = contents["key_pairs"]["sandbox_1"]
        except KeyError as e:
            self.fail(
                f"The ebay_rest.json file is missing an essential application, user or header record {e}"
            )

        # use good credentials
        try:
            # demonstrate one way to supply dicts during instantiation
            api = API(
                application=application, user=user, header=header, key_pair=key_pair
            )
        except Error as error:
            self.fail(f"Error {error.number} is {error.reason}  {error.detail}.\n")
        else:
            self.assertIsInstance(api, API, "An API object was not returned.")

        # try each credential that when bad should raise an exception
        sensitive_params = {
            # "application": ("placeholder",),
            "user": ("refresh_token_expiry",),
            "header": (
                "accept_language",
                "content_language",
                "country",
                "currency",
                "marketplace_id",
            ),
            # "key_pair": ("placeholder",),
        }
        good_config = {
            "user": user,
            "application": application,
            "header": header,
            "key_pair": key_pair,
        }
        for key in sensitive_params:
            for sub_key in sensitive_params[key]:
                bad_config = deepcopy(good_config)
                bad_config[key][sub_key] += "@bad"
                with self.assertRaises(
                    Error,
                    msg=f"No error raised on {key} {sub_key} with value {bad_config[key][sub_key]}",
                ):
                    # demonstrate another way to supply dicts during instantiation
                    API(**bad_config)

    def test_object_reuse(self):
        """Do the same parameters return the same API object?"""
        a1 = API(application="sandbox_1", user="sandbox_1", header="ENCA")
        a2 = API(application="sandbox_1", user="sandbox_1", header="ENCA")
        b = API(application="sandbox_1", user="sandbox_1", header="GB")
        self.assertEqual(a1, a2)
        self.assertNotEqual(a1, b)

    def test_credential_path(self):
        """supply a path to ebay_rest.json"""
        try:
            api = API(
                path=os.getcwd(), application="sandbox_1", user="sandbox_1", header="US"
            )
        except Error as error:
            self.fail(f"Error {error.number} is {error.reason}  {error.detail}.\n")
        else:
            self.assertIsInstance(api, API, "An API object was not returned.")

    def test_shipping_accuracy(self):
        """Is closer shipping less expensive?"""

        # domestic
        # d_market = 'EBAY_CA'    # set in header
        d_country = "CA"
        d_currency = "CAD"
        # d_zip = 'K1M 1M4'         # set in header

        # foreign
        # f_market = 'EBAY_GB'      # set in header
        f_country = "GB"
        # f_currency = 'GBP'
        # f_zip = 'SW1A 1AA'        # set in header

        # more constants
        shipping_currency = "USD"

        tally = 0  # how many times domestic shipping is less expensive than foreign

        d_api = API(application="production_1", user="production_1", header="ENCA")
        f_api = API(application="production_1", user="production_1", header="GB")

        # in the local marketplace, find items located locally that also ship to foreign
        # low-priced items are targeted because they are unlikely to have free shipping
        # black items are targeted because they are plentiful, and the q parameter is mandatory
        try:
            filter_ = (
                f"deliveryCountry:{f_country},itemLocationCountry:{d_country},price:[1..2],"
                f"priceCurrency:{d_currency} "
            )

            for record in d_api.buy_browse_search(q="black", filter=filter_, limit=10):
                if "record" not in record:
                    self.assertTrue(
                        "total" in record, f"Unexpected non-record{record}."
                    )
                else:
                    item = record["record"]
                    item_id = item["item_id"]
                    if item["item_location"]["country"] == d_country:
                        # get the lowest domestic shipping cost
                        try:
                            d_item = d_api.buy_browse_get_item(
                                item_id=item_id, fieldgroups="PRODUCT"
                            )
                        except Error as error:
                            self.fail(
                                f"Error {error.number} is {error.reason}  {error.detail}.\n"
                            )
                        d_shipping = self._get_lowest_shipping(
                            d_item, d_country, shipping_currency
                        )

                        # get the lowest foreign shipping cost
                        try:
                            f_item = f_api.buy_browse_get_item(
                                item_id=item_id, fieldgroups="PRODUCT"
                            )
                        except Error as error:
                            self.fail(
                                f"Error {error.number} is {error.reason}  {error.detail}.\n"
                            )
                        f_shipping = self._get_lowest_shipping(
                            f_item, f_country, shipping_currency
                        )

                        # compare costs, inconclusive when equal or some None
                        if (d_shipping is not None) and (f_shipping is not None):
                            if d_shipping < f_shipping:
                                tally += 1  # what we would expect most of the time
                            elif d_shipping > f_shipping:
                                tally -= (
                                    1  # very rare, put a break point here find trouble
                                )
                            else:
                                tally = (
                                    tally  # flat rate worldwide shipping is possible
                                )
                        else:
                            pass  # self.fail(f"For {item_id} both shipping costs can't be found.")
                    else:
                        self.fail(
                            f"Item {item_id} is supposed to be located in {d_country}."
                        )

        except Error as error:
            self.fail(f"Error {error.number} is {error.reason}  {error.detail}.\n")

        self.assertTrue(tally > 0, "Domestic shipping should cost less than foreign.")

    def _get_lowest_shipping(self, item: dict, country: str, currency: str):
        costs = []

        # is shipping excluded?
        ship_to_locations = item["ship_to_locations"]
        if ship_to_locations:
            if self._in_region(ship_to_locations["region_excluded"] or [], country):
                return None
            # if not in either list, then eBay concludes that shipping is allowed, so skip the following
            # elif not self._in_region(ship_to_locations['region_included'], country):
            # return None

        # find all costs
        if not item["shipping_options"]:
            return None
        for shipping_option in item["shipping_options"]:
            shipping_cost = shipping_option["shipping_cost"]
            if shipping_cost["currency"] == currency:
                cost = float(shipping_cost["value"])
            elif shipping_cost["converted_from_currency"] == currency:
                cost = float(shipping_cost["converted_from_value"])
            else:
                cost = self.currency_converter.convert(
                    float(shipping_cost["value"]), shipping_cost["currency"], currency
                )
            if cost is not None:
                costs.append(cost)
            else:
                self.fail("A shipping cost, without a cost, should not be possible.")

        if costs:
            costs.sort()
            return costs[0]
        else:
            self.fail("Shipping options without any options, should not be possible.")

    def _in_region(self, regions: dict, country: str):
        if country in ("CA", "US"):
            region_ids = ("AMERICAS", "NORTH_AMERICA")
        elif country in ("GB",):
            region_ids = ("EUROPE",)
        else:
            self.fail("add region ids for your country")

        for region in regions:
            region_type = region["region_type"]
            if region_type == "COUNTRY":
                if region["region_id"] == country:
                    return True
            elif region_type == "WORLD_REGION":
                if region["region_id"] in region_ids:
                    return True
            elif region_type == "WORLDWIDE":
                return True
        return False


class APISandboxSingleSiteTests(unittest.TestCase):
    """API tests that can be done on a single marketplace and in the sandbox.
    Also, try the non-default option of making asynchronous HTTP requests.
    """

    @classmethod
    def setUpClass(cls):
        try:
            # TODO Change to async_req=True.
            cls._api = API(
                application="sandbox_1", user="sandbox_1", header="US", async_req=False
            )
        except Error as error:
            cls.number = error.number
            cls.reason = error.reason
            cls.detail = error.detail
        else:
            cls.number = None
        cls.marketplace_id = cls._api._header["marketplace_id"]

    def setUp(self):
        if self.number is not None:
            self.fail(f"Error {self.number} is {self.reason}  {self.detail}.\n")

    def test_paging_no_results(self):
        try:
            for record in self._api.buy_browse_search(
                q="atomic-donkey-kick--this-will-not-be-found-Geraldine"
            ):
                if "record" not in record:
                    self.assertTrue(
                        "total" in record, f"Unexpected non-record{record}."
                    )
                    self.assertEqual(record["total"]["records_yielded"], 0)
                    self.assertEqual(record["total"]["records_available"], 0)
                else:
                    item = record["record"]
                    self.assertTrue(isinstance(item["item_id"], str), "Malformed item.")
                    self.fail(msg="No items should be returned.")
        except Error as error:
            self.fail(f"Error {error.number} is {error.reason}  {error.detail}.\n")

    def test_positional_none_kw_none(self):
        """Try a call with no positional arguments and no keyword arguments."""
        try:
            result = self._api.sell_compliance_get_listing_violations_summary(
                x_ebay_c_marketplace_id=self.marketplace_id
            )
        except Error as error:
            self.fail(f"Error {error.number} is {error.reason}  {error.detail}.\n")
        else:
            msg = "A call with zero positional and no kw arguments failed."
            self.assertIsNotNone("violation_summaries" in result, msg=msg)

    def test_positional_some_kw_none(self):
        # https://developer.ebay.com/api-docs/commerce/taxonomy/resources/category_tree/methods/getDefaultCategoryTreeId
        try:
            result = self._api.commerce_taxonomy_get_default_category_tree_id("EBAY_US")
        except Error as error:
            self.fail(f"Error {error.number} is {error.reason}  {error.detail}.\n")
        else:
            self.assertTrue(
                "category_tree_id" in result,
                msg="A call with some positional and no kw arguments failed.",
            )

    def test_positional_none_kw_some(self):
        try:
            for record in self._api.buy_browse_search(q="red"):
                if "record" in record:
                    item = record["record"]
                    self.assertTrue(
                        "item_id" in item,
                        msg="A call with some positional and no kw arguments failed.",
                    )
                    break
                else:
                    self.assertTrue(
                        "total" in record, f"Unexpected non-record{record}."
                    )
        except Error as error:
            self.fail(f"Error {error.number} is {error.reason}  {error.detail}.\n")

    def test_positional_some_kw_some(self):
        # This is redundant, several unit tests call buy_browse_search, so this test case is well covered.
        pass

    def test_try_except_else_api(self):
        """Test that an exception occurs when expected."""
        try:
            self._api.buy_browse_get_item(item_id="invalid")
        except Error as error:
            self.assertIsNotNone(
                f"Error {error.number} is {error.reason} with detail {error.detail}."
            )
        else:
            self.assertIsNotNone(None, msg="Failed to raise an exception.")

    def test_buying_options(self):
        """Does buying option filtering work?"""
        options = [
            "BEST_OFFER",
            "FIXED_PRICE",
        ]  # on Production 'AUCTION' and 'CLASSIFIED_AD' are also options
        for option in options:
            try:
                success = None
                keywords = "black"
                filter_ = "buyingOptions:{" + option + "}"
                for record in self._api.buy_browse_search(
                    q=keywords, filter=filter_, limit=500
                ):
                    if "record" not in record:
                        self.assertTrue(
                            "total" in record, f"Unexpected non-record{record}."
                        )
                    else:
                        item = record["record"]
                        if option in item["buying_options"]:
                            success = True
                        else:
                            success = False
                            break
            except Error as error:
                self.fail(f"Error {error.number} is {error.reason}  {error.detail}.\n")
            else:
                message = (
                    "The search for "
                    + option
                    + " items returned no items or a non-auction item."
                )
                self.assertTrue(success, message)

    def test_get_transaction_summary(self):
        """
        https://developer.ebay.com/api-docs/sell/finances/resources/transaction/methods/getTransactionSummary
        """
        try:
            result = self._api.sell_finances_get_transaction_summary(
                filter="transactionStatus:{PAYOUT}",
                x_ebay_c_marketplace_id=self.marketplace_id,
            )
        except Error as error:
            self.fail(f"Error {error.number} is {error.reason}  {error.detail}.\n")
        else:
            self.assertTrue("credit_count" in result)

    def test_sell_feed_create_inventory_task(self):
        """
        https://developer.ebay.com/api-docs/sell/feed/resources/inventory_task/methods/createInventoryTask

        The underlying REST call returns the new id in the header, which can't be gotten with the combination of Swagger
        and OpenAPI contract which we rely on here. A workaround uses a related call to retrieve task_id_new.
        """

        try:
            feed_type = "LMS_ACTIVE_INVENTORY_REPORT"
            body = {"schemaVersion": "1.0", "feedType": feed_type}
            task_ids_pre = self.get_task_ids(feed_type=feed_type)

            result = self._api.sell_feed_create_inventory_task(
                content_type="application/json", body=body
            )
            self.assertIsNone(
                result,
                "eBay made a change, the call now returns something, can the work around go?",
            )
            task_ids_post = self.get_task_ids(feed_type=feed_type)
            task_ids_new = task_ids_post.difference(task_ids_pre)
            self.assertEqual(
                1,
                len(task_ids_new),
                "In this test, one and only one task id is expected.",
            )
            # task_id_new = task_ids_new.pop()    # the new task id that we worked so hard to get

        except Error as error:
            self.fail(f"Error {error.number} is {error.reason}  {error.detail}.\n")
        except Exception as ex:
            self.fail(f"An unexpected error occurred: {str(ex)}")

    def get_task_ids(self, feed_type: string):
        """
        https://developer.ebay.com/api-docs/sell/feed/resources/inventory_task/methods/getInventoryTasks

        :param feed_type: (string)
        :return: task_ids (set(string))
        """
        task_ids = set()
        for record in self._api.sell_feed_get_inventory_tasks(
            feed_type=feed_type, look_back_days="1"
        ):
            if "record" in record:
                task_ids.add(record["record"]["task_id"])
            elif "total" in record:
                self.assertEqual(
                    record["total"]["records_yielded"],
                    record["total"]["records_available"],
                    "We missed some data - narrow the search!",
                )
            else:
                self.fail(f"unexpected record {record}")
        return task_ids

    def get_task_ids(self, feed_type: string):
        """
        https://developer.ebay.com/api-docs/sell/feed/resources/inventory_task/methods/getInventoryTasks

        :param feed_type: (string)
        :return: task_ids (set(string))
        """
        task_ids = set()
        self._api.sell_account_fulfeed_get_inventory_tasks()
        for record in self._api.sell_feed_get_inventory_tasks(
            feed_type=feed_type, look_back_days="1"
        ):
            if "record" in record:
                task_ids.add(record["record"]["task_id"])
            elif "total" in record:
                self.assertEqual(
                    record["total"]["records_yielded"],
                    record["total"]["records_available"],
                    "We missed some data - narrow the search!",
                )
            else:
                self.fail(f"unexpected record {record}")
        return task_ids


class APIProductionSingleTests(unittest.TestCase):
    """API tests that can be done on a single production marketplace."""

    @classmethod
    def setUpClass(cls):
        try:
            cls._api = API(application="production_1", user="production_1", header="US")
        except Error as error:
            cls.number = error.number
            cls.reason = error.reason
            cls.detail = error.detail
        else:
            cls.number = None

    def setUp(self):
        if self.number is not None:
            self.fail(f"Error {self.number} is {self.reason}  {self.detail}.\n")

    def test_complex_filtering(self):
        """Does complex filtering work?
        Use the production environment because it has vastly more items to work with
        and some filters are not available in the sandbox."""
        filters = []
        buying_option = "FIXED_PRICE"  # options are 'AUCTION', 'BEST_OFFER', 'CLASSIFIED_AD', 'FIXED_PRICE'
        filters.append("buyingOptions:{" + buying_option + "}")
        condition_option = "New"
        filters.append("itemCondition:{" + condition_option + "}")
        currency = "USD"
        filters.append(f"priceCurrency:{currency}")
        price_max = "50"
        filters.append("price:[.." + price_max + "]")
        # TODO add more filters https://developer.ebay.com/api-docs/buy/static/ref-buy-browse-filters.html
        filter_ = ",".join(filters)

        limit = 25
        try:
            for record in self._api.buy_browse_search(
                q="black", filter=filter_, limit=limit
            ):
                if "record" in record:
                    item = record["record"]
                    self.assertIn(
                        buying_option, item["buying_options"], "Wrong buying option."
                    )
                    if "AUCTION" in item["buying_options"]:
                        self.assertIsInstance(
                            item["item_end_date"],
                            str,
                            "Auctions must have an end date.",
                        )
                    # TODO eBay appears to consider some conditions equivalent so this assert is needs to change
                    # self.assertEqual(condition_option, item["condition"], "Wrong condition.")
                    if item["price"] is None:
                        self.assertIn(
                            "AUCTION",
                            item["buying_options"],
                            "Non-auctions need prices.",
                        )
                    else:
                        # The following relies on priceCurrency being in the filter.
                        self.assertLessEqual(
                            float(item["price"]["value"]),
                            float(price_max),
                            "Wrong price.",
                        )
                elif "total" in record:
                    self.assertEqual(
                        limit, record["total"]["records_yielded"], "Insufficient items."
                    )
                else:
                    self.fail(f"Unexpected record {record}")
        except Error as error:
            self.fail(f"Error {error.number} is {error.reason}  {error.detail}.")

    def test_hidden_page_boundaries(self):
        """
        Warning, this can't be run in the sandbox because searches do not return enough results.
        """
        limit_and_keywords = (
            (
                1,
                "red",
            ),  # start at one because zero is not allowed
            (
                2,
                "orange",
            ),
            (
                198,
                "yellow",
            ),
            (
                199,
                "green",
            ),
            (
                200,
                "blue",
            ),  # the biggest page can hold 200 items
            (
                201,
                "indigo",
            ),
            (
                202,
                "violet",
            ),
        )
        for limit, keywords in limit_and_keywords:
            counter = 0
            safety = limit + 5
            for record in self._api.buy_browse_search(limit=limit, q=keywords):
                if "record" not in record:
                    self.assertTrue(
                        "total" in record, f"Unexpected non-record{record}."
                    )
                else:
                    item = record["record"]
                    self.assertTrue(isinstance(item["item_id"], str), "Malformed item.")
                    counter += 1
                    if counter >= safety:
                        break
            self.assertTrue(counter == limit, f"Page boundary error on limit {limit}.")

    def test_sell_inventory(self):
        """
        See https://developer.ebay.com/api-docs/sell/inventory/static/overview.html

        :return:
        """
        # A unique, merchant-defined key (ID) for an inventory location.
        # This unique identifier, or key, is used in other Inventory API calls to identify an inventory location.
        length = random.randint(1, 36)
        allowed = (
            string.ascii_letters + string.digits + "_" + "-"
        )  # more might be allowed, eBay's docs are unclear
        merchant_location_key = "".join(random.choice(allowed) for _i in range(length))

        # Create a new inventory location.
        # https://developer.ebay.com/api-docs/sell/inventory/resources/location/methods/createInventoryLocation
        # A template for supplying all possible information about a location.
        _body = {
            "location": {
                "address": {
                    "addressLine1": "string",
                    "addressLine2": "string",
                    "city": "string",
                    "country": "CountryCodeEnum",
                    "county": "string",
                    "postalCode": "string",
                    "stateOrProvince": "string",
                },
                "geoCoordinates": {"latitude": "number", "longitude": "number"},
            },
            "locationAdditionalInformation": "string",
            "locationInstructions": "string",
            "locationTypes": ["StoreTypeEnum"],
            "locationWebUrl": "string",
            "merchantLocationStatus": "StatusEnum : [DISABLED,ENABLED]",
            "name": "string",
            "operatingHours": [
                {
                    "dayOfWeekEnum": "DayOfWeekEnum : [MONDAY,TUESDAY,WEDNESDAY,THURSDAY,FRIDAY,SATURDAY,SUNDAY]",
                    "intervals": [{"close": "string", "open": "string"}],
                }
            ],
            "phone": "string",
            "specialHours": [
                {"date": "string", "intervals": [{"close": "string", "open": "string"}]}
            ],
        }
        # A template for supplying the bare minimum about a location.
        body = {
            "location": {
                "address": {
                    "addressLine1": "2********e",
                    "addressLine2": "B********3",
                    "city": "S*****e",
                    "stateOrProvince": "**",
                    "postalCode": "9***5",
                    "country": "US",
                }
            },
            "locationInstructions": "Items ship from here.",
            "name": "W********1",
            "merchantLocationStatus": "ENABLED",
            "locationTypes": ["WAREHOUSE"],
        }
        try:
            self._api.sell_inventory_create_inventory_location(
                content_type="application/json",
                body=body,
                merchant_location_key=merchant_location_key,
            )
        except Error as error:
            self.fail(f"Error {error.number} is {error.reason}  {error.detail}.\n")

        # Confirm the new location.
        # https://developer.ebay.com/api-docs/sell/inventory/resources/location/methods/getInventoryLocations
        confirmed = False
        try:
            for record in self._api.sell_inventory_get_inventory_locations():
                if "record" not in record:
                    self.assertTrue(
                        "total" in record, f"Unexpected non-record{record}."
                    )
                else:
                    location = record["record"]
                    if "merchant_location_key" in location:
                        if location["merchant_location_key"] == merchant_location_key:
                            confirmed = True
                            break
                    else:
                        self.fail(
                            f"A location is missing the merchant_location_key {record}."
                        )
        except Error as error:
            self.fail(f"Error {error.number} is {error.reason}  {error.detail}.\n")
        self.assertTrue(confirmed, "Failed to confirm that the new location exists.")

    def test_sell_create_or_replace_inventory_item(self):
        """
        See https://developer.ebay.com/api-docs/sell/inventory/static/overview.html &
        https://developer.ebay.com/api-docs/sell/inventory/resources/inventory_item/methods/createOrReplaceInventoryItem#h2-samples

        :return:
        """
        # Create a new inventory item.
        # A template for supplying all possible information about an inventory item.
        _body = {
            "availability": {
                "pickupAtLocationAvailability": [
                    {
                        "availabilityType": "AvailabilityTypeEnum : [IN_STOCK,OUT_OF_STOCK,SHIP_TO_STORE]",
                        "fulfillmentTime": {
                            "unit": "TimeDurationUnitEnum : [YEAR,MONTH,DAY,HOUR,CALENDAR_DAY,"
                            "BUSINESS_DAY,MINUTE,SECOND,MILLISECOND]",
                            "value": "integer",
                        },
                        "merchantLocationKey": "string",
                        "quantity": "integer",
                    }
                ],
                "shipToLocationAvailability": {
                    "availabilityDistributions": [
                        {
                            "fulfillmentTime": {
                                "unit": "TimeDurationUnitEnum : [YEAR,MONTH,DAY,HOUR,"
                                "CALENDAR_DAY,BUSINESS_DAY,MINUTE,SECOND,MILLISECOND]",
                                # noqa: E501
                                "value": "integer",
                            },
                            "merchantLocationKey": "string",
                            "quantity": "integer",
                        }
                    ],
                    "quantity": "integer",
                },
            },
            "condition": "ConditionEnum : [NEW,LIKE_NEW,NEW_OTHER,"
            "NEW_WITH_DEFECTS,MANUFACTURER_REFURBISHED,"
            "CERTIFIED_REFURBISHED,EXCELLENT_REFURBISHED,"
            "VERY_GOOD_REFURBISHED,GOOD_REFURBISHED,SELLER_REFURBISHED,"
            "USED_EXCELLENT,USED_VERY_GOOD,USED_GOOD,USED_ACCEPTABLE,FOR_PARTS_OR_NOT_WORKING]",
            # noqa: E501
            "conditionDescription": "string",
            "packageWeightAndSize": {
                "dimensions": {
                    "height": "number",
                    "length": "number",
                    "unit": "LengthUnitOfMeasureEnum : [INCH,FEET,CENTIMETER,METER]",
                    "width": "number",
                },
                "packageType": "PackageTypeEnum : [LETTER,BULKY_GOODS,CARAVAN,CARS,"
                "EUROPALLET,EXPANDABLE_TOUGH_BAGS,EXTRA_LARGE_PACK,"  # noqa: E501
                "FURNITURE,INDUSTRY_VEHICLES,LARGE_CANADA_POSTBOX,"
                "LARGE_CANADA_POST_BUBBLE_MAILER,LARGE_ENVELOPE,"
                "MAILING_BOX,MEDIUM_CANADA_POST_BOX,MEDIUM_CANADA_POST_BUBBLE_MAILER,"
                "MOTORBIKES,ONE_WAY_PALLET,PACKAGE_THICK_ENVELOPE,PADDED_BAGS,"
                "PARCEL_OR_PADDED_ENVELOPE,ROLL,SMALL_CANADA_POST_BOX,SMALL_CANADA_POST_BUBBLE_MAILER,"
                "TOUGH_BAGS,UPS_LETTER,USPS_FLAT_RATE_ENVELOPE,USPS_LARGE_PACK,VERY_LARGE_PACK,"
                "WINE_PAK]",
                "weight": {
                    "unit": "WeightUnitOfMeasureEnum : [POUND,KILOGRAM,OUNCE,GRAM]",
                    "value": "number",
                },
            },
            "product": {
                "aspects": "string",
                "brand": "string",
                "description": "string",
                "ean": ["string"],
                "epid": "string",
                "imageUrls": ["string"],
                "isbn": ["string"],
                "mpn": "string",
                "subtitle": "string",
                "title": "string",
                "upc": ["string"],
                "videoIds": ["string"],
            },
        }
        # A template for supplying limited information about an inventory item.
        body = {
            "availability": {"shipToLocationAvailability": {"quantity": 50}},
            "condition": "NEW",
            "product": {
                "title": "GoPro Hero4 Helmet Cam",
                "description": "New GoPro Hero4 Helmet Cam. Unopened box.",
                "imageUrls": [
                    "https://i.ebayimg.com/images/i/182196556219-0-1/s-l1000.jpg",
                    "https://i.ebayimg.com/images/i/182196556219-0-1/s-l1001.jpg",
                    "https://i.ebayimg.com/images/i/182196556219-0-1/s-l1002.jpg",
                ],
            },
        }

        # The natural language used in the body; see 'Local Support' in the following link for permitted values.
        # https://developer.ebay.com/api-docs/static/rest-request-components.html#HTTP
        content_language = "en-US"

        # The seller-defined SKU value for the inventory item, unique across the seller's inventory.
        length = random.randint(1, 50)
        allowed = (
            string.ascii_letters + string.digits + "_" + "-"
        )  # more might be allowed, eBay's docs are unclear
        sku = "".join(random.choice(allowed) for _i in range(length))

        try:
            self._api.sell_inventory_create_or_replace_inventory_item(
                body=body,
                content_language=content_language,
                content_type="application/json",
                sku=sku,
            )
        except Error as error:
            self.fail(f"Error {error.number} is {error.reason}  {error.detail}.\n")

    @unittest.skip  # TODO Finish the unit test and then attempt to relocate it to the APISandboxSingleSiteTests class.
    def test_file_upload(self):
        """
        Test uploading a file. Relevant eBay documentation:
        https://developer.ebay.com/api-docs/sell/feed/resources/task/methods/createTask
        https://developer.ebay.com/api-docs/sell/feed/resources/task/methods/uploadFile.
        """
        body = {
            "feedType": "LMS_ORDER_ACK",  # TODO Is this the correct feed type for use with sell_feed_upload_file?.
            "schemaVersion": "1.0",
        }
        try:
            self._api.sell_feed_create_task(
                body=body,
                content_type="application/json",
                x_ebay_c_marketplace_id="placeholder",
            )
        except Error as error:
            self.fail(f"Error {error.number} is {error.reason}  {error.detail}.\n")
        else:
            # TODO Somehow, get the task id from the sell_feed_create_task response header. Example from the header.
            # location:https://api.ebay.com/sell/feed/v1/task/task-11-1271150479
            # As of 2022-10-22, the Swagger code generated from eBay's OpenAPI contract doesn't support this.
            task_id = "task-11-1271150479"

            file_name = "test_file_upload.json"

            # TODO Somehow pass along the path and file name for the file to be uploaded.
            path_and_file_name = self._config_location = os.path.join(
                os.getcwd(), "test_file_upload.json"
            )
            # Lines like "local_var_files = {}" are found in src/ebay_rest/api/sell_feed/api/task_api.py
            # Instead of an empty dictionary, how do we make this happen?
            local_var_files = {"filename", path_and_file_name}
            # The Swagger code generated from eBay's OpenAPI contract doesn't support this. Checked on 2022-10-22.

            try:
                self._api.sell_feed_upload_file(
                    task_id=task_id,
                    file_name=file_name,
                    content_type="application/json",
                )
            except Error as error:
                self.fail(f"Error {error.number} is {error.reason}  {error.detail}.\n")

    @unittest.skip  # TODO finish it
    def test_marketplace_account_deletion(self):
        """
        See https://developer.ebay.com/marketplace-account-deletion
        and https://developer.ebay.com/api-docs/commerce/notification/overview.html.

        # AsyncAPI specification
        # https://developer.ebay.com/cms/files/asyncapi/marketplace_account_deletion.yaml
        # https://www.asyncapi.com
        # https://github.com/dedoussis/asynction
        # https://github.com/dutradda/asyncapi-python

        :return:
        """
        # TODO get this from ebay_rest.json
        end_point = "https://e***********4.s*************.com/notification-endpoint"
        alert_email = "**@**.com"

        # The verification token has to be between 32 and 80 characters,
        # and allowed characters include alphanumeric characters, underscore (_),  and hyphen (-).
        # No other characters are allowed.
        # verification_token = "7*******-d***-***c-b***-***********a"
        length = random.randint(32, 80)
        allowed = string.ascii_letters + string.digits + "_" + "-"
        verification_token = "".join(random.choice(allowed) for _i in range(length))

        topic_id = "MARKETPLACE_ACCOUNT_DELETION"

        # to confirm that the desired topic is available, eBay's recommended workflow does not require this
        try:
            result = self._api.commerce_notification_get_topics()
        except Error as error:
            self.fail(f"Error {error.number} is {error.reason}  {error.detail}.\n")
        else:
            success = False
            key = "topics"
            if key in result:
                for topic in result[key]:
                    if topic["topic_id"] == topic_id:
                        if topic["status"] == "ENABLED":
                            success = True
                            break
            self.assertTrue(success)

        # ensure that an alert email is correctly configured
        try:
            result = self._api.commerce_notification_get_config()
        except Error as error:
            if error.reason != "Not Found":
                self.fail(f"Error {error.number} is {error.reason}  {error.detail}.\n")
            else:
                try:
                    config = {"alertEmail": alert_email}
                    self._api.commerce_notification_update_config(
                        body=config, content_type="application/json"
                    )
                except Error as error:
                    self.fail(
                        f"Error {error.number} is {error.reason}  {error.detail}.\n"
                    )
        else:
            self.assertEqual(result["alert_email"], alert_email)

        # Ensure that a destination is correctly configured.
        # Note: The destination should be created and ready to respond with the expected
        # challengeResponse for the endpoint to be registered successfully.
        # Refer to the Notification API Overview for more information.
        # https://developer.ebay.com/api-docs/commerce/notification/overview.html
        try:
            result = self._api.commerce_notification_get_destinations()
        except Error as error:
            self.fail(f"Error {error.number} is {error.reason}  {error.detail}.\n")
        else:
            if result["total"] == 0:
                try:
                    destination_request = {
                        "status": "ENABLED",
                        "deliveryConfig": {
                            "endpoint": end_point,
                            "verificationToken": verification_token,
                        },
                    }
                    self._api.commerce_notification_create_destination(
                        body=destination_request, content_type="application/json"
                    )
                except Error as error:
                    self.fail(
                        f"Error {error.number} is {error.reason}  {error.detail}.\n"
                    )
                else:
                    pass  # TODO
            else:
                pass  # TODO

        # ensure subscribed
        # TODO

        # get notices
        # description: 'The Account Deletion payload.'
        #     username, string, The username for the user.
        #     userId, string, The immutable public userId for the user
        #     eiasToken, string, The legacy eiasToken specific to the user
        try:
            result = self._api.commerce_notification_get_topic(topic_id)
        except Error as error:
            self.fail(f"Error {error.number} is {error.reason}  {error.detail}.\n")
        else:
            pass  # TODO
            self.assertTrue("Subscribe" not in result["description"])


class APISandboxDigitalSignatureTests(unittest.TestCase):
    """API tests that test eBay Digital Signatures and public/private key pairs"""

    @classmethod
    def setUpClass(cls):
        sandbox = True
        application = "sandbox_1" if sandbox else "production_1"
        user = "sandbox_1" if sandbox else "production_1"
        key_pair = "sandbox_1" if sandbox else "production_1"
        _api = API(
            application=application,
            user=user,
            header="US",
            key_pair=key_pair,
            digital_signatures=True,
        )
        original_private_key = _api._key_pair_token._private_key
        original_signing_key_id = _api._key_pair_token._signing_key_id
        if original_private_key and original_signing_key_id:
            cls.key_fail = None
        else:
            cls.key_fail = (
                "Need a private key and signing_key_id for Digital Signatures tests"
            )
        # retain only what the class methods need
        cls.application = application
        cls._api = _api
        cls.original_private_key = original_private_key
        cls.original_signing_key_id = original_signing_key_id
        cls.user = user
        cls.marketplace_id = _api._header["marketplace_id"]

    def setUp(self):
        # Reset private key
        self._api._key_pair_token._private_key = self.original_private_key
        if self.key_fail is not None:
            self.fail(self.key_fail)

    def test_001_get_signing_key(self):
        """Check that we can load all the remaining key information
        given the private key and signing_key_id.
        """
        key_pair = {
            "private_key": self.original_private_key,
            "signing_key_id": self.original_signing_key_id,
        }
        api = API(
            application=self.application,
            user=self.user,
            header="US",
            key_pair=key_pair,
            digital_signatures=True,
        )
        try:
            key = api.get_digital_signature_key(create_new=False)
        except Error as error:
            self.fail(f"Error {error.number} is {error.reason}  {error.detail}.\n")
        else:
            self.assertEqual(key["private_key"], self.original_private_key)
            self.assertEqual(key["signing_key_id"], self.original_signing_key_id)

    def test_002_make_call_with_signature(self):
        """Check that we can make a call using the Digital Signature"""
        try:
            result = self._api.sell_finances_get_transaction_summary(
                filter="transactionStatus:{PAYOUT}",
                x_ebay_c_marketplace_id=self.marketplace_id,
            )
        except Error as error:
            self.fail(f"Error {error.number} is {error.reason}  {error.detail}.\n")
        else:
            self.assertTrue("credit_count" in result)
            # Break the private key and try again
            # This is an example key from draft-ietf-httpbis-message-signatures-15
            self._api._key_pair_token._private_key = (
                "MC4CAQAwBQYDK2VwBCIEIJ+DYvh6SEqVTm50DFtMDoQikTmiCqirVv9mWG9qfSnF"
            )
            try:
                self._api.sell_finances_get_transaction_summary(
                    filter="transactionStatus:{PAYOUT}",
                    x_ebay_c_marketplace_id=self.marketplace_id,
                )
            except Error as error:
                self.assertEqual(error.number, 99403)
                # Check for eBay 'Signature validation failed' error
                ebay_error = loads(error.detail)
                signature_validation_failed = (215120, 215121, 215122)
                self.assertTrue(
                    ebay_error["errors"][0]["errorId"] in signature_validation_failed
                )
            else:
                self.fail("Did not fail with invalid private key!")

    # This test permanently creates a new public/private key pair which cannot
    # be removed, hence it is skipped by default.
    @unittest.skip
    def test_003_create_signing_key(self):
        """Check that we can create a new public/private key."""
        self._api._key_pair_token._private_key = None
        self._api._key_pair_token._signing_key_id = None
        with self.assertRaises(Error):
            _key = self._api.get_digital_signature_key(create_new=False)
        key = self._api.get_digital_signature_key(create_new=True)
        self.assertTrue(key["private_key"])
        self.assertNotEqual(key["private_key"], self.original_private_key)
        try:
            result = self._api.sell_finances_get_transaction_summary(
                filter="transactionStatus:{PAYOUT}",
                x_ebay_c_marketplace_id=self.marketplace_id,
            )
        except Error as error:
            self.fail(f"Error {error.number} is {error.reason}  {error.detail}.\n")
        else:
            self.assertTrue("credit_count" in result)


class DateTimeTests(unittest.TestCase):

    def test_now_returns_datetime_with_utc(self):
        dt = DateTime.now()
        self.assertIsInstance(
            dt, datetime.datetime, "now() should return a datetime instance."
        )
        self.assertIsNotNone(dt.tzinfo, "Returned datetime should have a timezone set.")
        # Instead of comparing to datetime.timezone.utc directly, verify that the UTC offset is zero.
        self.assertEqual(
            dt.tzinfo.utcoffset(dt),
            datetime.timedelta(0),
            "Returned datetime should have an offset of zero (UTC).",
        )

    def test_to_string_format(self):
        dt = datetime.datetime(
            2025, 2, 4, 12, 30, 15, 123456, tzinfo=datetime.timezone.utc
        )
        s = DateTime.to_string(dt)
        pattern = r"\d{4}-\d{2}-\d{2}T\d{2}:\d{2}:\d{2}\.\d{3}Z"
        self.assertRegex(
            s, pattern, "to_string() output did not match the expected format."
        )

    def test_from_string_returns_correct_datetime(self):
        s = "2025-02-04T12:30:15.123Z"
        dt = DateTime.from_string(s)
        expected = datetime.datetime(
            2025, 2, 4, 12, 30, 15, 123000, tzinfo=datetime.timezone.utc
        )
        self.assertEqual(
            dt, expected, "from_string() did not return the expected datetime."
        )

    def test_round_trip_conversion(self):
        original = datetime.datetime(
            2025, 2, 4, 12, 30, 15, 789123, tzinfo=datetime.timezone.utc
        )
        s = DateTime.to_string(original)
        converted = DateTime.from_string(s)
        expected_microsecond = (original.microsecond // 1000) * 1000
        expected = datetime.datetime(
            original.year,
            original.month,
            original.day,
            original.hour,
            original.minute,
            original.second,
            expected_microsecond,
            tzinfo=datetime.timezone.utc,
        )
        self.assertEqual(
            converted,
            expected,
            "Round-trip conversion did not yield the expected datetime.",
        )

    def test_to_string_known_value(self):
        dt = datetime.datetime(
            2004, 8, 4, 19, 9, 2, 768000, tzinfo=datetime.timezone.utc
        )
        result = DateTime.to_string(dt)
        expected = "2004-08-04T19:09:02.768Z"
        self.assertEqual(
            result,
            expected,
            "to_string() did not produce the expected output for a known value.",
        )

    def test_to_string_with_invalid_input(self):
        with self.assertRaises(Error) as context:
            DateTime.to_string("not a datetime")  # type: ignore
        self.assertEqual(
            context.exception.number,
            98001,
            "to_string() did not raise the expected error for non-datetime input.",
        )

    def test_from_string_with_invalid_input_type(self):
        with self.assertRaises(Error) as context:
            DateTime.from_string(12345)  # type: ignore
        self.assertEqual(
            context.exception.number,
            98003,
            "from_string() did not raise the expected error for non-string input.",
        )

    def test_from_string_with_invalid_format(self):
        with self.assertRaises(Error) as context:
            DateTime.from_string("invalid date format")
        self.assertEqual(
            context.exception.number,
            98004,
            "from_string() did not raise the expected error for an invalid format.",
        )

    def test_now_increasing(self):
        dt1 = DateTime.now()
        dt2 = DateTime.now()
        self.assertLessEqual(
            dt1, dt2, "Consecutive calls to now() should yield non-decreasing times."
        )


class TokenTests(unittest.TestCase):
    def test_get_authorization_code(self):
        """
        Notes:
        1. A web browser window should pop up while this test successfully runs.
        2. eBay also refers to the authorization code as a token refresh token.
        """
        try:
            # The user sand_box_2 should be a copy of sand_box_1 without refresh token values.
            api = API(application="sandbox_1", user="sandbox_2", header="US")
        except Error as error:
            self.fail(f"Error {error.number} is {error.reason}  {error.detail}.\n")
        else:
            # Try a method that needs a user refresh token.
            try:
                result = api.sell_compliance_get_listing_violations_summary(
                    x_ebay_c_marketplace_id=api._header["marketplace_id"]
                )
            except Error as error:
                self.fail(f"Error {error.number} is {error.reason}  {error.detail}.\n")
            else:
                # Do something that makes use of the result to avoid an IDE warning about not using the variable.
                self.assertIsNotNone("violation_summaries" in result)


class MultitonTests(unittest.TestCase):
    def test_help(self):
        """Documentation from the wrapped class should be returned instead of Multiton."""
        doc_string = API.__doc__
        self.assertTrue(
            "Multiton" not in doc_string and "API" in doc_string,
            msg="Double-check functools.update_wrapper in multiton.py.",
        )


class ReferenceTests(unittest.TestCase):
    def test_get_application_scopes(self):
        self.assertIsNotNone(
            Reference.get_application_scopes(), msg="Failed to load application scopes."
        )

    def test_get_country_codes(self):
        self.assertIsNotNone(
            Reference.get_country_codes(), msg="Failed to load country codes."
        )

    def test_get_currency_codes(self):
        self.assertIsNotNone(
            Reference.get_currency_codes(), msg="Failed to load currency codes."
        )

    def test_global_id_values(self):
        self.assertIsNotNone(
            Reference.get_global_id_values(), msg="Failed to load global id values."
        )

    def test_marketplace_id_values(self):
        self.assertIsNotNone(
            Reference.get_marketplace_id_values(),
            msg="Failed to load marketplace id values.",
        )

    def test_get_user_scopes(self):
        self.assertIsNotNone(
            Reference.get_user_scopes(), msg="Failed to load user scopes."
        )


if __name__ == "__main__":
    unittest.main()



# coding: utf-8

"""
    EDIS public shipping API

    <div class=\"msgbox_important\"><p class=\"msgbox_importantInDiv\" data-mc-autonum=\"&lt;b&gt;&lt;span style=&quot;color: #dd1e31;&quot; class=&quot;mcFormatColor&quot;&gt;Important! &lt;/span&gt;&lt;/b&gt;\"><span class=\"autonumber\"><span><b><span style=\"color: #dd1e31;\" class=\"mcFormatColor\">Important!</span></b></span></span> This method is only available for Greater-China based sellers with an active eDIS account.</p></div><br>This API allows 3rd party developers in the Greater-China area to process package shipping details.  # noqa: E501

    OpenAPI spec version: 1.0.0
    
    Generated by: https://github.com/swagger-api/swagger-codegen.git
"""

import pprint
import re  # noqa: F401

import six

class BatteryQualResponses(object):
    """NOTE: This class is auto generated by the swagger code generator program.

    Do not edit the class manually.
    """
    """
    Attributes:
      swagger_types (dict): The key is attribute name
                            and the value is attribute type.
      attribute_map (dict): The key is attribute name
                            and the value is json key in definition.
    """
    swagger_types = {
        'battery_type': 'str',
        'elec_qualification_id': 'str',
        'elec_qualification_name': 'str',
        'expire_date': 'str',
        'remark': 'str'
    }

    attribute_map = {
        'battery_type': 'batteryType',
        'elec_qualification_id': 'elecQualificationId',
        'elec_qualification_name': 'elecQualificationName',
        'expire_date': 'expireDate',
        'remark': 'remark'
    }

    def __init__(self, battery_type=None, elec_qualification_id=None, elec_qualification_name=None, expire_date=None, remark=None):  # noqa: E501
        """BatteryQualResponses - a model defined in Swagger"""  # noqa: E501
        self._battery_type = None
        self._elec_qualification_id = None
        self._elec_qualification_name = None
        self._expire_date = None
        self._remark = None
        self.discriminator = None
        if battery_type is not None:
            self.battery_type = battery_type
        if elec_qualification_id is not None:
            self.elec_qualification_id = elec_qualification_id
        if elec_qualification_name is not None:
            self.elec_qualification_name = elec_qualification_name
        if expire_date is not None:
            self.expire_date = expire_date
        if remark is not None:
            self.remark = remark

    @property
    def battery_type(self):
        """Gets the battery_type of this BatteryQualResponses.  # noqa: E501

        The type of the battery. For implementation help, refer to <a href='https://developer.ebay.com/api-docs/sell/edelivery_international_shipping/types/api:LiBatteryTypeEnum'>eBay API documentation</a>  # noqa: E501

        :return: The battery_type of this BatteryQualResponses.  # noqa: E501
        :rtype: str
        """
        return self._battery_type

    @battery_type.setter
    def battery_type(self, battery_type):
        """Sets the battery_type of this BatteryQualResponses.

        The type of the battery. For implementation help, refer to <a href='https://developer.ebay.com/api-docs/sell/edelivery_international_shipping/types/api:LiBatteryTypeEnum'>eBay API documentation</a>  # noqa: E501

        :param battery_type: The battery_type of this BatteryQualResponses.  # noqa: E501
        :type: str
        """

        self._battery_type = battery_type

    @property
    def elec_qualification_id(self):
        """Gets the elec_qualification_id of this BatteryQualResponses.  # noqa: E501

        The unique identifier of the battery's electric qualification, which indicates that the battery has a certificate of approval for global shipping.  # noqa: E501

        :return: The elec_qualification_id of this BatteryQualResponses.  # noqa: E501
        :rtype: str
        """
        return self._elec_qualification_id

    @elec_qualification_id.setter
    def elec_qualification_id(self, elec_qualification_id):
        """Sets the elec_qualification_id of this BatteryQualResponses.

        The unique identifier of the battery's electric qualification, which indicates that the battery has a certificate of approval for global shipping.  # noqa: E501

        :param elec_qualification_id: The elec_qualification_id of this BatteryQualResponses.  # noqa: E501
        :type: str
        """

        self._elec_qualification_id = elec_qualification_id

    @property
    def elec_qualification_name(self):
        """Gets the elec_qualification_name of this BatteryQualResponses.  # noqa: E501

        The name of the battery's electric qualification.  # noqa: E501

        :return: The elec_qualification_name of this BatteryQualResponses.  # noqa: E501
        :rtype: str
        """
        return self._elec_qualification_name

    @elec_qualification_name.setter
    def elec_qualification_name(self, elec_qualification_name):
        """Sets the elec_qualification_name of this BatteryQualResponses.

        The name of the battery's electric qualification.  # noqa: E501

        :param elec_qualification_name: The elec_qualification_name of this BatteryQualResponses.  # noqa: E501
        :type: str
        """

        self._elec_qualification_name = elec_qualification_name

    @property
    def expire_date(self):
        """Gets the expire_date of this BatteryQualResponses.  # noqa: E501

        The expiration date of the battery qualification.  # noqa: E501

        :return: The expire_date of this BatteryQualResponses.  # noqa: E501
        :rtype: str
        """
        return self._expire_date

    @expire_date.setter
    def expire_date(self, expire_date):
        """Sets the expire_date of this BatteryQualResponses.

        The expiration date of the battery qualification.  # noqa: E501

        :param expire_date: The expire_date of this BatteryQualResponses.  # noqa: E501
        :type: str
        """

        self._expire_date = expire_date

    @property
    def remark(self):
        """Gets the remark of this BatteryQualResponses.  # noqa: E501

        A remark to the seller themselves.<br><br><span class=\"tablenote\"><b>Note:</b> This field is always returned, but will show as null or have an empty value if not defined/applicable.</span>  # noqa: E501

        :return: The remark of this BatteryQualResponses.  # noqa: E501
        :rtype: str
        """
        return self._remark

    @remark.setter
    def remark(self, remark):
        """Sets the remark of this BatteryQualResponses.

        A remark to the seller themselves.<br><br><span class=\"tablenote\"><b>Note:</b> This field is always returned, but will show as null or have an empty value if not defined/applicable.</span>  # noqa: E501

        :param remark: The remark of this BatteryQualResponses.  # noqa: E501
        :type: str
        """

        self._remark = remark

    def to_dict(self):
        """Returns the model properties as a dict"""
        result = {}

        for attr, _ in six.iteritems(self.swagger_types):
            value = getattr(self, attr)
            if isinstance(value, list):
                result[attr] = list(map(
                    lambda x: x.to_dict() if hasattr(x, "to_dict") else x,
                    value
                ))
            elif hasattr(value, "to_dict"):
                result[attr] = value.to_dict()
            elif isinstance(value, dict):
                result[attr] = dict(map(
                    lambda item: (item[0], item[1].to_dict())
                    if hasattr(item[1], "to_dict") else item,
                    value.items()
                ))
            else:
                result[attr] = value
        if issubclass(BatteryQualResponses, dict):
            for key, value in self.items():
                result[key] = value

        return result

    def to_str(self):
        """Returns the string representation of the model"""
        return pprint.pformat(self.to_dict())

    def __repr__(self):
        """For `print` and `pprint`"""
        return self.to_str()

    def __eq__(self, other):
        """Returns true if both objects are equal"""
        if not isinstance(other, BatteryQualResponses):
            return False

        return self.__dict__ == other.__dict__

    def __ne__(self, other):
        """Returns true if both objects are not equal"""
        return not self == other



# coding: utf-8

"""
    Vero Public API's

    The VeRO API has methods that allow owners of an intellectual property to report eBay listings that infringe on their intellectual property rights. It contains methods to report a listing, retrieve supported VeRO reason codes, and retrieve the status of submitted and items.  # noqa: E501

    OpenAPI spec version: 1.0.0
    
    Generated by: https://github.com/swagger-api/swagger-codegen.git
"""

import pprint
import re  # noqa: F401

import six

class ReportStatusResponse(object):
    """NOTE: This class is auto generated by the swagger code generator program.

    Do not edit the class manually.
    """
    """
    Attributes:
      swagger_types (dict): The key is attribute name
                            and the value is attribute type.
      attribute_map (dict): The key is attribute name
                            and the value is json key in definition.
    """
    swagger_types = {
        'reported_item_details': 'list[ReportedItem]',
        'vero_report_id': 'str',
        'vero_report_status': 'str'
    }

    attribute_map = {
        'reported_item_details': 'reportedItemDetails',
        'vero_report_id': 'veroReportId',
        'vero_report_status': 'veroReportStatus'
    }

    def __init__(self, reported_item_details=None, vero_report_id=None, vero_report_status=None):  # noqa: E501
        """ReportStatusResponse - a model defined in Swagger"""  # noqa: E501
        self._reported_item_details = None
        self._vero_report_id = None
        self._vero_report_status = None
        self.discriminator = None
        if reported_item_details is not None:
            self.reported_item_details = reported_item_details
        if vero_report_id is not None:
            self.vero_report_id = vero_report_id
        if vero_report_status is not None:
            self.vero_report_status = vero_report_status

    @property
    def reported_item_details(self):
        """Gets the reported_item_details of this ReportStatusResponse.  # noqa: E501

        This array returns the status for each eBay item that was included as part of the report.<br><br>This array is only returned if the <b>includeItemDetails</b> query parameter is set to <code>true</code>.  # noqa: E501

        :return: The reported_item_details of this ReportStatusResponse.  # noqa: E501
        :rtype: list[ReportedItem]
        """
        return self._reported_item_details

    @reported_item_details.setter
    def reported_item_details(self, reported_item_details):
        """Sets the reported_item_details of this ReportStatusResponse.

        This array returns the status for each eBay item that was included as part of the report.<br><br>This array is only returned if the <b>includeItemDetails</b> query parameter is set to <code>true</code>.  # noqa: E501

        :param reported_item_details: The reported_item_details of this ReportStatusResponse.  # noqa: E501
        :type: list[ReportedItem]
        """

        self._reported_item_details = reported_item_details

    @property
    def vero_report_id(self):
        """Gets the vero_report_id of this ReportStatusResponse.  # noqa: E501

        The unique packet identifier of the VeRO report.  # noqa: E501

        :return: The vero_report_id of this ReportStatusResponse.  # noqa: E501
        :rtype: str
        """
        return self._vero_report_id

    @vero_report_id.setter
    def vero_report_id(self, vero_report_id):
        """Sets the vero_report_id of this ReportStatusResponse.

        The unique packet identifier of the VeRO report.  # noqa: E501

        :param vero_report_id: The vero_report_id of this ReportStatusResponse.  # noqa: E501
        :type: str
        """

        self._vero_report_id = vero_report_id

    @property
    def vero_report_status(self):
        """Gets the vero_report_status of this ReportStatusResponse.  # noqa: E501

        The processing status of the requested VeRO report. For implementation help, refer to <a href='https://developer.ebay.com/api-docs/commerce/vero/types/VeroReportStatusEnum'>eBay API documentation</a>  # noqa: E501

        :return: The vero_report_status of this ReportStatusResponse.  # noqa: E501
        :rtype: str
        """
        return self._vero_report_status

    @vero_report_status.setter
    def vero_report_status(self, vero_report_status):
        """Sets the vero_report_status of this ReportStatusResponse.

        The processing status of the requested VeRO report. For implementation help, refer to <a href='https://developer.ebay.com/api-docs/commerce/vero/types/VeroReportStatusEnum'>eBay API documentation</a>  # noqa: E501

        :param vero_report_status: The vero_report_status of this ReportStatusResponse.  # noqa: E501
        :type: str
        """

        self._vero_report_status = vero_report_status

    def to_dict(self):
        """Returns the model properties as a dict"""
        result = {}

        for attr, _ in six.iteritems(self.swagger_types):
            value = getattr(self, attr)
            if isinstance(value, list):
                result[attr] = list(map(
                    lambda x: x.to_dict() if hasattr(x, "to_dict") else x,
                    value
                ))
            elif hasattr(value, "to_dict"):
                result[attr] = value.to_dict()
            elif isinstance(value, dict):
                result[attr] = dict(map(
                    lambda item: (item[0], item[1].to_dict())
                    if hasattr(item[1], "to_dict") else item,
                    value.items()
                ))
            else:
                result[attr] = value
        if issubclass(ReportStatusResponse, dict):
            for key, value in self.items():
                result[key] = value

        return result

    def to_str(self):
        """Returns the string representation of the model"""
        return pprint.pformat(self.to_dict())

    def __repr__(self):
        """For `print` and `pprint`"""
        return self.to_str()

    def __eq__(self, other):
        """Returns true if both objects are equal"""
        if not isinstance(other, ReportStatusResponse):
            return False

        return self.__dict__ == other.__dict__

    def __ne__(self, other):
        """Returns true if both objects are not equal"""
        return not self == other



# coding: utf-8

"""
    Vero Public API's

    The VeRO API has methods that allow owners of an intellectual property to report eBay listings that infringe on their intellectual property rights. It contains methods to report a listing, retrieve supported VeRO reason codes, and retrieve the status of submitted and items.  # noqa: E501

    OpenAPI spec version: 1.0.0
    
    Generated by: https://github.com/swagger-api/swagger-codegen.git
"""

import pprint
import re  # noqa: F401

import six

class VeroReportItemsResponse(object):
    """NOTE: This class is auto generated by the swagger code generator program.

    Do not edit the class manually.
    """
    """
    Attributes:
      swagger_types (dict): The key is attribute name
                            and the value is attribute type.
      attribute_map (dict): The key is attribute name
                            and the value is json key in definition.
    """
    swagger_types = {
        'vero_report_id': 'str',
        'vero_report_status': 'str'
    }

    attribute_map = {
        'vero_report_id': 'veroReportId',
        'vero_report_status': 'veroReportStatus'
    }

    def __init__(self, vero_report_id=None, vero_report_status=None):  # noqa: E501
        """VeroReportItemsResponse - a model defined in Swagger"""  # noqa: E501
        self._vero_report_id = None
        self._vero_report_status = None
        self.discriminator = None
        if vero_report_id is not None:
            self.vero_report_id = vero_report_id
        if vero_report_status is not None:
            self.vero_report_status = vero_report_status

    @property
    def vero_report_id(self):
        """Gets the vero_report_id of this VeroReportItemsResponse.  # noqa: E501

        The unique identifier of the VeRO report that the item violations have been added to.<br><br><span class=\"tablenote\"><b>Note:</b> Users should keep track of the returned <b>veroReportId</b>, as there is no programmatic way to retrieve this value at this time.</span>  # noqa: E501

        :return: The vero_report_id of this VeroReportItemsResponse.  # noqa: E501
        :rtype: str
        """
        return self._vero_report_id

    @vero_report_id.setter
    def vero_report_id(self, vero_report_id):
        """Sets the vero_report_id of this VeroReportItemsResponse.

        The unique identifier of the VeRO report that the item violations have been added to.<br><br><span class=\"tablenote\"><b>Note:</b> Users should keep track of the returned <b>veroReportId</b>, as there is no programmatic way to retrieve this value at this time.</span>  # noqa: E501

        :param vero_report_id: The vero_report_id of this VeroReportItemsResponse.  # noqa: E501
        :type: str
        """

        self._vero_report_id = vero_report_id

    @property
    def vero_report_status(self):
        """Gets the vero_report_status of this VeroReportItemsResponse.  # noqa: E501

        The processing status of the VeRO report. For implementation help, refer to <a href='https://developer.ebay.com/api-docs/commerce/vero/types/VeroReportStatusEnum'>eBay API documentation</a>  # noqa: E501

        :return: The vero_report_status of this VeroReportItemsResponse.  # noqa: E501
        :rtype: str
        """
        return self._vero_report_status

    @vero_report_status.setter
    def vero_report_status(self, vero_report_status):
        """Sets the vero_report_status of this VeroReportItemsResponse.

        The processing status of the VeRO report. For implementation help, refer to <a href='https://developer.ebay.com/api-docs/commerce/vero/types/VeroReportStatusEnum'>eBay API documentation</a>  # noqa: E501

        :param vero_report_status: The vero_report_status of this VeroReportItemsResponse.  # noqa: E501
        :type: str
        """

        self._vero_report_status = vero_report_status

    def to_dict(self):
        """Returns the model properties as a dict"""
        result = {}

        for attr, _ in six.iteritems(self.swagger_types):
            value = getattr(self, attr)
            if isinstance(value, list):
                result[attr] = list(map(
                    lambda x: x.to_dict() if hasattr(x, "to_dict") else x,
                    value
                ))
            elif hasattr(value, "to_dict"):
                result[attr] = value.to_dict()
            elif isinstance(value, dict):
                result[attr] = dict(map(
                    lambda item: (item[0], item[1].to_dict())
                    if hasattr(item[1], "to_dict") else item,
                    value.items()
                ))
            else:
                result[attr] = value
        if issubclass(VeroReportItemsResponse, dict):
            for key, value in self.items():
                result[key] = value

        return result

    def to_str(self):
        """Returns the string representation of the model"""
        return pprint.pformat(self.to_dict())

    def __repr__(self):
        """For `print` and `pprint`"""
        return self.to_str()

    def __eq__(self, other):
        """Returns true if both objects are equal"""
        if not isinstance(other, VeroReportItemsResponse):
            return False

        return self.__dict__ == other.__dict__

    def __ne__(self, other):
        """Returns true if both objects are not equal"""
        return not self == other



# coding: utf-8

"""
    Metadata API

    The Metadata API has operations that retrieve configuration details pertaining to the different eBay marketplaces. In addition to marketplace information, the API also has operations that get information that helps sellers list items on eBay.  # noqa: E501

    OpenAPI spec version: v1.11.0
    
    Generated by: https://github.com/swagger-api/swagger-codegen.git
"""

import pprint
import re  # noqa: F401

import six

class ListingDuration(object):
    """NOTE: This class is auto generated by the swagger code generator program.

    Do not edit the class manually.
    """
    """
    Attributes:
      swagger_types (dict): The key is attribute name
                            and the value is attribute type.
      attribute_map (dict): The key is attribute name
                            and the value is json key in definition.
    """
    swagger_types = {
        'duration_values': 'list[str]',
        'listing_type': 'str'
    }

    attribute_map = {
        'duration_values': 'durationValues',
        'listing_type': 'listingType'
    }

    def __init__(self, duration_values=None, listing_type=None):  # noqa: E501
        """ListingDuration - a model defined in Swagger"""  # noqa: E501
        self._duration_values = None
        self._listing_type = None
        self.discriminator = None
        if duration_values is not None:
            self.duration_values = duration_values
        if listing_type is not None:
            self.listing_type = listing_type

    @property
    def duration_values(self):
        """Gets the duration_values of this ListingDuration.  # noqa: E501

        This array defines the supported time duration options available for the listing type.  # noqa: E501

        :return: The duration_values of this ListingDuration.  # noqa: E501
        :rtype: list[str]
        """
        return self._duration_values

    @duration_values.setter
    def duration_values(self, duration_values):
        """Sets the duration_values of this ListingDuration.

        This array defines the supported time duration options available for the listing type.  # noqa: E501

        :param duration_values: The duration_values of this ListingDuration.  # noqa: E501
        :type: list[str]
        """

        self._duration_values = duration_values

    @property
    def listing_type(self):
        """Gets the listing_type of this ListingDuration.  # noqa: E501

        The enumerated value returned in this field indicates the listing type for the duration value(s). For implementation help, refer to <a href='https://developer.ebay.com/api-docs/sell/metadata/types/sel:ListingTypeEnum'>eBay API documentation</a>  # noqa: E501

        :return: The listing_type of this ListingDuration.  # noqa: E501
        :rtype: str
        """
        return self._listing_type

    @listing_type.setter
    def listing_type(self, listing_type):
        """Sets the listing_type of this ListingDuration.

        The enumerated value returned in this field indicates the listing type for the duration value(s). For implementation help, refer to <a href='https://developer.ebay.com/api-docs/sell/metadata/types/sel:ListingTypeEnum'>eBay API documentation</a>  # noqa: E501

        :param listing_type: The listing_type of this ListingDuration.  # noqa: E501
        :type: str
        """

        self._listing_type = listing_type

    def to_dict(self):
        """Returns the model properties as a dict"""
        result = {}

        for attr, _ in six.iteritems(self.swagger_types):
            value = getattr(self, attr)
            if isinstance(value, list):
                result[attr] = list(map(
                    lambda x: x.to_dict() if hasattr(x, "to_dict") else x,
                    value
                ))
            elif hasattr(value, "to_dict"):
                result[attr] = value.to_dict()
            elif isinstance(value, dict):
                result[attr] = dict(map(
                    lambda item: (item[0], item[1].to_dict())
                    if hasattr(item[1], "to_dict") else item,
                    value.items()
                ))
            else:
                result[attr] = value
        if issubclass(ListingDuration, dict):
            for key, value in self.items():
                result[key] = value

        return result

    def to_str(self):
        """Returns the string representation of the model"""
        return pprint.pformat(self.to_dict())

    def __repr__(self):
        """For `print` and `pprint`"""
        return self.to_str()

    def __eq__(self, other):
        """Returns true if both objects are equal"""
        if not isinstance(other, ListingDuration):
            return False

        return self.__dict__ == other.__dict__

    def __ne__(self, other):
        """Returns true if both objects are not equal"""
        return not self == other



# coding: utf-8

"""
    Vero Public API's

    The VeRO API has methods that allow owners of an intellectual property to report eBay listings that infringe on their intellectual property rights. It contains methods to report a listing, retrieve supported VeRO reason codes, and retrieve the status of submitted and items.  # noqa: E501

    OpenAPI spec version: 1.0.0
    
    Generated by: https://github.com/swagger-api/swagger-codegen.git
"""

import pprint
import re  # noqa: F401

import six

class ReasonCodeDetailType(object):
    """NOTE: This class is auto generated by the swagger code generator program.

    Do not edit the class manually.
    """
    """
    Attributes:
      swagger_types (dict): The key is attribute name
                            and the value is attribute type.
      attribute_map (dict): The key is attribute name
                            and the value is json key in definition.
    """
    swagger_types = {
        'brief_text': 'str',
        'detailed_text': 'str',
        'vero_reason_code_id': 'str'
    }

    attribute_map = {
        'brief_text': 'briefText',
        'detailed_text': 'detailedText',
        'vero_reason_code_id': 'veroReasonCodeId'
    }

    def __init__(self, brief_text=None, detailed_text=None, vero_reason_code_id=None):  # noqa: E501
        """ReasonCodeDetailType - a model defined in Swagger"""  # noqa: E501
        self._brief_text = None
        self._detailed_text = None
        self._vero_reason_code_id = None
        self.discriminator = None
        if brief_text is not None:
            self.brief_text = brief_text
        if detailed_text is not None:
            self.detailed_text = detailed_text
        if vero_reason_code_id is not None:
            self.vero_reason_code_id = vero_reason_code_id

    @property
    def brief_text(self):
        """Gets the brief_text of this ReasonCodeDetailType.  # noqa: E501

        The short description of the infringement associated with the reason code.  # noqa: E501

        :return: The brief_text of this ReasonCodeDetailType.  # noqa: E501
        :rtype: str
        """
        return self._brief_text

    @brief_text.setter
    def brief_text(self, brief_text):
        """Sets the brief_text of this ReasonCodeDetailType.

        The short description of the infringement associated with the reason code.  # noqa: E501

        :param brief_text: The brief_text of this ReasonCodeDetailType.  # noqa: E501
        :type: str
        """

        self._brief_text = brief_text

    @property
    def detailed_text(self):
        """Gets the detailed_text of this ReasonCodeDetailType.  # noqa: E501

        The long description of the infringement associated with the reason code.  # noqa: E501

        :return: The detailed_text of this ReasonCodeDetailType.  # noqa: E501
        :rtype: str
        """
        return self._detailed_text

    @detailed_text.setter
    def detailed_text(self, detailed_text):
        """Sets the detailed_text of this ReasonCodeDetailType.

        The long description of the infringement associated with the reason code.  # noqa: E501

        :param detailed_text: The detailed_text of this ReasonCodeDetailType.  # noqa: E501
        :type: str
        """

        self._detailed_text = detailed_text

    @property
    def vero_reason_code_id(self):
        """Gets the vero_reason_code_id of this ReasonCodeDetailType.  # noqa: E501

        The unique identifier of a VeRO reason code.  # noqa: E501

        :return: The vero_reason_code_id of this ReasonCodeDetailType.  # noqa: E501
        :rtype: str
        """
        return self._vero_reason_code_id

    @vero_reason_code_id.setter
    def vero_reason_code_id(self, vero_reason_code_id):
        """Sets the vero_reason_code_id of this ReasonCodeDetailType.

        The unique identifier of a VeRO reason code.  # noqa: E501

        :param vero_reason_code_id: The vero_reason_code_id of this ReasonCodeDetailType.  # noqa: E501
        :type: str
        """

        self._vero_reason_code_id = vero_reason_code_id

    def to_dict(self):
        """Returns the model properties as a dict"""
        result = {}

        for attr, _ in six.iteritems(self.swagger_types):
            value = getattr(self, attr)
            if isinstance(value, list):
                result[attr] = list(map(
                    lambda x: x.to_dict() if hasattr(x, "to_dict") else x,
                    value
                ))
            elif hasattr(value, "to_dict"):
                result[attr] = value.to_dict()
            elif isinstance(value, dict):
                result[attr] = dict(map(
                    lambda item: (item[0], item[1].to_dict())
                    if hasattr(item[1], "to_dict") else item,
                    value.items()
                ))
            else:
                result[attr] = value
        if issubclass(ReasonCodeDetailType, dict):
            for key, value in self.items():
                result[key] = value

        return result

    def to_str(self):
        """Returns the string representation of the model"""
        return pprint.pformat(self.to_dict())

    def __repr__(self):
        """For `print` and `pprint`"""
        return self.to_str()

    def __eq__(self, other):
        """Returns true if both objects are equal"""
        if not isinstance(other, ReasonCodeDetailType):
            return False

        return self.__dict__ == other.__dict__

    def __ne__(self, other):
        """Returns true if both objects are not equal"""
        return not self == other



# coding: utf-8

"""
    Metadata API

    The Metadata API has operations that retrieve configuration details pertaining to the different eBay marketplaces. In addition to marketplace information, the API also has operations that get information that helps sellers list items on eBay.  # noqa: E501

    OpenAPI spec version: v1.11.0
    
    Generated by: https://github.com/swagger-api/swagger-codegen.git
"""

import pprint
import re  # noqa: F401

import six

class GetCurrenciesResponse(object):
    """NOTE: This class is auto generated by the swagger code generator program.

    Do not edit the class manually.
    """
    """
    Attributes:
      swagger_types (dict): The key is attribute name
                            and the value is attribute type.
      attribute_map (dict): The key is attribute name
                            and the value is json key in definition.
    """
    swagger_types = {
        'default_currency': 'Currency',
        'marketplace_id': 'str'
    }

    attribute_map = {
        'default_currency': 'defaultCurrency',
        'marketplace_id': 'marketplaceId'
    }

    def __init__(self, default_currency=None, marketplace_id=None):  # noqa: E501
        """GetCurrenciesResponse - a model defined in Swagger"""  # noqa: E501
        self._default_currency = None
        self._marketplace_id = None
        self.discriminator = None
        if default_currency is not None:
            self.default_currency = default_currency
        if marketplace_id is not None:
            self.marketplace_id = marketplace_id

    @property
    def default_currency(self):
        """Gets the default_currency of this GetCurrenciesResponse.  # noqa: E501


        :return: The default_currency of this GetCurrenciesResponse.  # noqa: E501
        :rtype: Currency
        """
        return self._default_currency

    @default_currency.setter
    def default_currency(self, default_currency):
        """Sets the default_currency of this GetCurrenciesResponse.


        :param default_currency: The default_currency of this GetCurrenciesResponse.  # noqa: E501
        :type: Currency
        """

        self._default_currency = default_currency

    @property
    def marketplace_id(self):
        """Gets the marketplace_id of this GetCurrenciesResponse.  # noqa: E501

        The ID of the eBay marketplace to which the default currency applies. For implementation help, refer to <a href='https://developer.ebay.com/api-docs/sell/metadata/types/bas:MarketplaceIdEnum'>eBay API documentation</a>  # noqa: E501

        :return: The marketplace_id of this GetCurrenciesResponse.  # noqa: E501
        :rtype: str
        """
        return self._marketplace_id

    @marketplace_id.setter
    def marketplace_id(self, marketplace_id):
        """Sets the marketplace_id of this GetCurrenciesResponse.

        The ID of the eBay marketplace to which the default currency applies. For implementation help, refer to <a href='https://developer.ebay.com/api-docs/sell/metadata/types/bas:MarketplaceIdEnum'>eBay API documentation</a>  # noqa: E501

        :param marketplace_id: The marketplace_id of this GetCurrenciesResponse.  # noqa: E501
        :type: str
        """

        self._marketplace_id = marketplace_id

    def to_dict(self):
        """Returns the model properties as a dict"""
        result = {}

        for attr, _ in six.iteritems(self.swagger_types):
            value = getattr(self, attr)
            if isinstance(value, list):
                result[attr] = list(map(
                    lambda x: x.to_dict() if hasattr(x, "to_dict") else x,
                    value
                ))
            elif hasattr(value, "to_dict"):
                result[attr] = value.to_dict()
            elif isinstance(value, dict):
                result[attr] = dict(map(
                    lambda item: (item[0], item[1].to_dict())
                    if hasattr(item[1], "to_dict") else item,
                    value.items()
                ))
            else:
                result[attr] = value
        if issubclass(GetCurrenciesResponse, dict):
            for key, value in self.items():
                result[key] = value

        return result

    def to_str(self):
        """Returns the string representation of the model"""
        return pprint.pformat(self.to_dict())

    def __repr__(self):
        """For `print` and `pprint`"""
        return self.to_str()

    def __eq__(self, other):
        """Returns true if both objects are equal"""
        if not isinstance(other, GetCurrenciesResponse):
            return False

        return self.__dict__ == other.__dict__

    def __ne__(self, other):
        """Returns true if both objects are not equal"""
        return not self == other



# coding: utf-8

"""
    Metadata API

    The Metadata API has operations that retrieve configuration details pertaining to the different eBay marketplaces. In addition to marketplace information, the API also has operations that get information that helps sellers list items on eBay.  # noqa: E501

    OpenAPI spec version: v1.11.0
    
    Generated by: https://github.com/swagger-api/swagger-codegen.git
"""

import pprint
import re  # noqa: F401

import six

class Currency(object):
    """NOTE: This class is auto generated by the swagger code generator program.

    Do not edit the class manually.
    """
    """
    Attributes:
      swagger_types (dict): The key is attribute name
                            and the value is attribute type.
      attribute_map (dict): The key is attribute name
                            and the value is json key in definition.
    """
    swagger_types = {
        'code': 'str',
        'description': 'str'
    }

    attribute_map = {
        'code': 'code',
        'description': 'description'
    }

    def __init__(self, code=None, description=None):  # noqa: E501
        """Currency - a model defined in Swagger"""  # noqa: E501
        self._code = None
        self._description = None
        self.discriminator = None
        if code is not None:
            self.code = code
        if description is not None:
            self.description = description

    @property
    def code(self):
        """Gets the code of this Currency.  # noqa: E501

        The three-letter <a href=\"https://www.iso.org/iso-4217-currency-codes.html \" target=\"_blank\">ISO 4217</a> code returned.  <br /><br /><b>Restriction: </b> Only the currency of the marketplace is supported. Examples: on the US marketplace, the only currency supported is the United States dollar, <code>USD</code>; on the Canadian marketplace, the only currency supported is the Canadian dollar, <code>CAD</code>. For implementation help, refer to <a href='https://developer.ebay.com/api-docs/sell/metadata/types/bas:CurrencyCodeEnum'>eBay API documentation</a>  # noqa: E501

        :return: The code of this Currency.  # noqa: E501
        :rtype: str
        """
        return self._code

    @code.setter
    def code(self, code):
        """Sets the code of this Currency.

        The three-letter <a href=\"https://www.iso.org/iso-4217-currency-codes.html \" target=\"_blank\">ISO 4217</a> code returned.  <br /><br /><b>Restriction: </b> Only the currency of the marketplace is supported. Examples: on the US marketplace, the only currency supported is the United States dollar, <code>USD</code>; on the Canadian marketplace, the only currency supported is the Canadian dollar, <code>CAD</code>. For implementation help, refer to <a href='https://developer.ebay.com/api-docs/sell/metadata/types/bas:CurrencyCodeEnum'>eBay API documentation</a>  # noqa: E501

        :param code: The code of this Currency.  # noqa: E501
        :type: str
        """

        self._code = code

    @property
    def description(self):
        """Gets the description of this Currency.  # noqa: E501

        The description of the returned three-letter code. For example, if the code is <code>USD</code>, the description returned would be <code>US Dollar</code>.  # noqa: E501

        :return: The description of this Currency.  # noqa: E501
        :rtype: str
        """
        return self._description

    @description.setter
    def description(self, description):
        """Sets the description of this Currency.

        The description of the returned three-letter code. For example, if the code is <code>USD</code>, the description returned would be <code>US Dollar</code>.  # noqa: E501

        :param description: The description of this Currency.  # noqa: E501
        :type: str
        """

        self._description = description

    def to_dict(self):
        """Returns the model properties as a dict"""
        result = {}

        for attr, _ in six.iteritems(self.swagger_types):
            value = getattr(self, attr)
            if isinstance(value, list):
                result[attr] = list(map(
                    lambda x: x.to_dict() if hasattr(x, "to_dict") else x,
                    value
                ))
            elif hasattr(value, "to_dict"):
                result[attr] = value.to_dict()
            elif isinstance(value, dict):
                result[attr] = dict(map(
                    lambda item: (item[0], item[1].to_dict())
                    if hasattr(item[1], "to_dict") else item,
                    value.items()
                ))
            else:
                result[attr] = value
        if issubclass(Currency, dict):
            for key, value in self.items():
                result[key] = value

        return result

    def to_str(self):
        """Returns the string representation of the model"""
        return pprint.pformat(self.to_dict())

    def __repr__(self):
        """For `print` and `pprint`"""
        return self.to_str()

    def __eq__(self, other):
        """Returns true if both objects are equal"""
        if not isinstance(other, Currency):
            return False

        return self.__dict__ == other.__dict__

    def __ne__(self, other):
        """Returns true if both objects are not equal"""
        return not self == other



# coding: utf-8

"""
    Metadata API

    The Metadata API has operations that retrieve configuration details pertaining to the different eBay marketplaces. In addition to marketplace information, the API also has operations that get information that helps sellers list items on eBay.  # noqa: E501

    OpenAPI spec version: v1.11.0
    
    Generated by: https://github.com/swagger-api/swagger-codegen.git
"""

import pprint
import re  # noqa: F401

import six

class CategoryPolicy(object):
    """NOTE: This class is auto generated by the swagger code generator program.

    Do not edit the class manually.
    """
    """
    Attributes:
      swagger_types (dict): The key is attribute name
                            and the value is attribute type.
      attribute_map (dict): The key is attribute name
                            and the value is json key in definition.
    """
    swagger_types = {
        'auto_pay_enabled': 'bool',
        'b2b_vat_enabled': 'bool',
        'category_id': 'str',
        'category_tree_id': 'str',
        'ean_support': 'str',
        'expired': 'bool',
        'intangible_enabled': 'bool',
        'isbn_support': 'str',
        'lsd': 'bool',
        'minimum_reserve_price': 'float',
        'orpa': 'bool',
        'orra': 'bool',
        'payment_methods': 'list[str]',
        'reduce_reserve_allowed': 'bool',
        'upc_support': 'str',
        'virtual': 'bool'
    }

    attribute_map = {
        'auto_pay_enabled': 'autoPayEnabled',
        'b2b_vat_enabled': 'b2bVatEnabled',
        'category_id': 'categoryId',
        'category_tree_id': 'categoryTreeId',
        'ean_support': 'eanSupport',
        'expired': 'expired',
        'intangible_enabled': 'intangibleEnabled',
        'isbn_support': 'isbnSupport',
        'lsd': 'lsd',
        'minimum_reserve_price': 'minimumReservePrice',
        'orpa': 'orpa',
        'orra': 'orra',
        'payment_methods': 'paymentMethods',
        'reduce_reserve_allowed': 'reduceReserveAllowed',
        'upc_support': 'upcSupport',
        'virtual': 'virtual'
    }

    def __init__(self, auto_pay_enabled=None, b2b_vat_enabled=None, category_id=None, category_tree_id=None, ean_support=None, expired=None, intangible_enabled=None, isbn_support=None, lsd=None, minimum_reserve_price=None, orpa=None, orra=None, payment_methods=None, reduce_reserve_allowed=None, upc_support=None, virtual=None):  # noqa: E501
        """CategoryPolicy - a model defined in Swagger"""  # noqa: E501
        self._auto_pay_enabled = None
        self._b2b_vat_enabled = None
        self._category_id = None
        self._category_tree_id = None
        self._ean_support = None
        self._expired = None
        self._intangible_enabled = None
        self._isbn_support = None
        self._lsd = None
        self._minimum_reserve_price = None
        self._orpa = None
        self._orra = None
        self._payment_methods = None
        self._reduce_reserve_allowed = None
        self._upc_support = None
        self._virtual = None
        self.discriminator = None
        if auto_pay_enabled is not None:
            self.auto_pay_enabled = auto_pay_enabled
        if b2b_vat_enabled is not None:
            self.b2b_vat_enabled = b2b_vat_enabled
        if category_id is not None:
            self.category_id = category_id
        if category_tree_id is not None:
            self.category_tree_id = category_tree_id
        if ean_support is not None:
            self.ean_support = ean_support
        if expired is not None:
            self.expired = expired
        if intangible_enabled is not None:
            self.intangible_enabled = intangible_enabled
        if isbn_support is not None:
            self.isbn_support = isbn_support
        if lsd is not None:
            self.lsd = lsd
        if minimum_reserve_price is not None:
            self.minimum_reserve_price = minimum_reserve_price
        if orpa is not None:
            self.orpa = orpa
        if orra is not None:
            self.orra = orra
        if payment_methods is not None:
            self.payment_methods = payment_methods
        if reduce_reserve_allowed is not None:
            self.reduce_reserve_allowed = reduce_reserve_allowed
        if upc_support is not None:
            self.upc_support = upc_support
        if virtual is not None:
            self.virtual = virtual

    @property
    def auto_pay_enabled(self):
        """Gets the auto_pay_enabled of this CategoryPolicy.  # noqa: E501

        If this field is returned as <code>true</code>, the corresponding category supports immediate payment for listings. The immediate payment feature is applicable to fixed-price listings, to auction listings with the 'Buy It Now' option enabled, and for a motor vehicle listing that requires an initial deposit. If the immediate payment feature is enabled for a listing, the buyer must pay immediately after clicking the 'Buy it Now' button. <p>This field is only returned when <code>true</code> (not returned when false).</p>  # noqa: E501

        :return: The auto_pay_enabled of this CategoryPolicy.  # noqa: E501
        :rtype: bool
        """
        return self._auto_pay_enabled

    @auto_pay_enabled.setter
    def auto_pay_enabled(self, auto_pay_enabled):
        """Sets the auto_pay_enabled of this CategoryPolicy.

        If this field is returned as <code>true</code>, the corresponding category supports immediate payment for listings. The immediate payment feature is applicable to fixed-price listings, to auction listings with the 'Buy It Now' option enabled, and for a motor vehicle listing that requires an initial deposit. If the immediate payment feature is enabled for a listing, the buyer must pay immediately after clicking the 'Buy it Now' button. <p>This field is only returned when <code>true</code> (not returned when false).</p>  # noqa: E501

        :param auto_pay_enabled: The auto_pay_enabled of this CategoryPolicy.  # noqa: E501
        :type: bool
        """

        self._auto_pay_enabled = auto_pay_enabled

    @property
    def b2b_vat_enabled(self):
        """Gets the b2b_vat_enabled of this CategoryPolicy.  # noqa: E501

        If this field is returned as <code>true</code>, the corresponding category supports business-to-business (B2B) VAT listings. If this field is not present, the category does not have B2B VAT listings. This feature is applicable to the eBay Germany (DE), Austria (AT), and Switzerland (CH) sites only.<p>This field is only returned when <code>true</code> (not returned when false).</p>  # noqa: E501

        :return: The b2b_vat_enabled of this CategoryPolicy.  # noqa: E501
        :rtype: bool
        """
        return self._b2b_vat_enabled

    @b2b_vat_enabled.setter
    def b2b_vat_enabled(self, b2b_vat_enabled):
        """Sets the b2b_vat_enabled of this CategoryPolicy.

        If this field is returned as <code>true</code>, the corresponding category supports business-to-business (B2B) VAT listings. If this field is not present, the category does not have B2B VAT listings. This feature is applicable to the eBay Germany (DE), Austria (AT), and Switzerland (CH) sites only.<p>This field is only returned when <code>true</code> (not returned when false).</p>  # noqa: E501

        :param b2b_vat_enabled: The b2b_vat_enabled of this CategoryPolicy.  # noqa: E501
        :type: bool
        """

        self._b2b_vat_enabled = b2b_vat_enabled

    @property
    def category_id(self):
        """Gets the category_id of this CategoryPolicy.  # noqa: E501

        The unique identifier of the eBay leaf category for which metadata is being returned.  # noqa: E501

        :return: The category_id of this CategoryPolicy.  # noqa: E501
        :rtype: str
        """
        return self._category_id

    @category_id.setter
    def category_id(self, category_id):
        """Sets the category_id of this CategoryPolicy.

        The unique identifier of the eBay leaf category for which metadata is being returned.  # noqa: E501

        :param category_id: The category_id of this CategoryPolicy.  # noqa: E501
        :type: str
        """

        self._category_id = category_id

    @property
    def category_tree_id(self):
        """Gets the category_tree_id of this CategoryPolicy.  # noqa: E501

        The unique identifier of the category tree.  # noqa: E501

        :return: The category_tree_id of this CategoryPolicy.  # noqa: E501
        :rtype: str
        """
        return self._category_tree_id

    @category_tree_id.setter
    def category_tree_id(self, category_tree_id):
        """Sets the category_tree_id of this CategoryPolicy.

        The unique identifier of the category tree.  # noqa: E501

        :param category_tree_id: The category_tree_id of this CategoryPolicy.  # noqa: E501
        :type: str
        """

        self._category_tree_id = category_tree_id

    @property
    def ean_support(self):
        """Gets the ean_support of this CategoryPolicy.  # noqa: E501

        This enumerated value indicates whether or not European Article Numbers (EANs) are supported/required when listing products in the category. For implementation help, refer to <a href='https://developer.ebay.com/api-docs/sell/metadata/types/sel:ProductIdentiferEnabledEnum'>eBay API documentation</a>  # noqa: E501

        :return: The ean_support of this CategoryPolicy.  # noqa: E501
        :rtype: str
        """
        return self._ean_support

    @ean_support.setter
    def ean_support(self, ean_support):
        """Sets the ean_support of this CategoryPolicy.

        This enumerated value indicates whether or not European Article Numbers (EANs) are supported/required when listing products in the category. For implementation help, refer to <a href='https://developer.ebay.com/api-docs/sell/metadata/types/sel:ProductIdentiferEnabledEnum'>eBay API documentation</a>  # noqa: E501

        :param ean_support: The ean_support of this CategoryPolicy.  # noqa: E501
        :type: str
        """

        self._ean_support = ean_support

    @property
    def expired(self):
        """Gets the expired of this CategoryPolicy.  # noqa: E501

        If this field is returned as <code>true</code>, the corresponding category is no longer a valid eBay category on the site, and items may not be listed in this category. You can use the <a href=\"/api-docs/sell/taxonomy/resources/category_tree/methods/getExpiredCategories\" target=\"_blank\">getExpiredCategories</a> method (of the <b>Taxonomy API</b>) to find the leaf category that replaced the expired category. <p>This field is only returned when <code>true</code> (not returned when false).</p>  # noqa: E501

        :return: The expired of this CategoryPolicy.  # noqa: E501
        :rtype: bool
        """
        return self._expired

    @expired.setter
    def expired(self, expired):
        """Sets the expired of this CategoryPolicy.

        If this field is returned as <code>true</code>, the corresponding category is no longer a valid eBay category on the site, and items may not be listed in this category. You can use the <a href=\"/api-docs/sell/taxonomy/resources/category_tree/methods/getExpiredCategories\" target=\"_blank\">getExpiredCategories</a> method (of the <b>Taxonomy API</b>) to find the leaf category that replaced the expired category. <p>This field is only returned when <code>true</code> (not returned when false).</p>  # noqa: E501

        :param expired: The expired of this CategoryPolicy.  # noqa: E501
        :type: bool
        """

        self._expired = expired

    @property
    def intangible_enabled(self):
        """Gets the intangible_enabled of this CategoryPolicy.  # noqa: E501

        If this field is returned as <code>true</code>, the category supports the listing of intangible goods or services.  # noqa: E501

        :return: The intangible_enabled of this CategoryPolicy.  # noqa: E501
        :rtype: bool
        """
        return self._intangible_enabled

    @intangible_enabled.setter
    def intangible_enabled(self, intangible_enabled):
        """Sets the intangible_enabled of this CategoryPolicy.

        If this field is returned as <code>true</code>, the category supports the listing of intangible goods or services.  # noqa: E501

        :param intangible_enabled: The intangible_enabled of this CategoryPolicy.  # noqa: E501
        :type: bool
        """

        self._intangible_enabled = intangible_enabled

    @property
    def isbn_support(self):
        """Gets the isbn_support of this CategoryPolicy.  # noqa: E501

        This enumerated value indicates whether or not International Standard Book Numbers (ISBNs) are supported/required when listing products in the specified category. For implementation help, refer to <a href='https://developer.ebay.com/api-docs/sell/metadata/types/sel:ProductIdentiferEnabledEnum'>eBay API documentation</a>  # noqa: E501

        :return: The isbn_support of this CategoryPolicy.  # noqa: E501
        :rtype: str
        """
        return self._isbn_support

    @isbn_support.setter
    def isbn_support(self, isbn_support):
        """Sets the isbn_support of this CategoryPolicy.

        This enumerated value indicates whether or not International Standard Book Numbers (ISBNs) are supported/required when listing products in the specified category. For implementation help, refer to <a href='https://developer.ebay.com/api-docs/sell/metadata/types/sel:ProductIdentiferEnabledEnum'>eBay API documentation</a>  # noqa: E501

        :param isbn_support: The isbn_support of this CategoryPolicy.  # noqa: E501
        :type: str
        """

        self._isbn_support = isbn_support

    @property
    def lsd(self):
        """Gets the lsd of this CategoryPolicy.  # noqa: E501

        If this field (Lot Size Disabled) is returned as <code>true</code>, the corresponding category does not support lot listings. A lot listing is a listing that features multiple related items that must be purchased by one buyer in one transaction. <p>This field is only returned when <code>true</code> (not returned when false).</p>  # noqa: E501

        :return: The lsd of this CategoryPolicy.  # noqa: E501
        :rtype: bool
        """
        return self._lsd

    @lsd.setter
    def lsd(self, lsd):
        """Sets the lsd of this CategoryPolicy.

        If this field (Lot Size Disabled) is returned as <code>true</code>, the corresponding category does not support lot listings. A lot listing is a listing that features multiple related items that must be purchased by one buyer in one transaction. <p>This field is only returned when <code>true</code> (not returned when false).</p>  # noqa: E501

        :param lsd: The lsd of this CategoryPolicy.  # noqa: E501
        :type: bool
        """

        self._lsd = lsd

    @property
    def minimum_reserve_price(self):
        """Gets the minimum_reserve_price of this CategoryPolicy.  # noqa: E501

        Indicates the Minimum Reserve Price for an auction listing in this category. If there is no Minimum Reserve Price, a value of <code>0.0</code> is returned in this field.  # noqa: E501

        :return: The minimum_reserve_price of this CategoryPolicy.  # noqa: E501
        :rtype: float
        """
        return self._minimum_reserve_price

    @minimum_reserve_price.setter
    def minimum_reserve_price(self, minimum_reserve_price):
        """Sets the minimum_reserve_price of this CategoryPolicy.

        Indicates the Minimum Reserve Price for an auction listing in this category. If there is no Minimum Reserve Price, a value of <code>0.0</code> is returned in this field.  # noqa: E501

        :param minimum_reserve_price: The minimum_reserve_price of this CategoryPolicy.  # noqa: E501
        :type: float
        """

        self._minimum_reserve_price = minimum_reserve_price

    @property
    def orpa(self):
        """Gets the orpa of this CategoryPolicy.  # noqa: E501

        If this field (Override Reserve Price Allowed) is returned as <code>true</code>, all leaf categories on the site allow the seller to completely remove the reserve price for an active auction listing. For information on removing or reducing reserve prices, see <a href=\"https://www.ebay.com/help/selling/listings/selling-auctions/reserve-prices?id=4143\"  target=\"_blank\">Setting a reserve price</a>.  # noqa: E501

        :return: The orpa of this CategoryPolicy.  # noqa: E501
        :rtype: bool
        """
        return self._orpa

    @orpa.setter
    def orpa(self, orpa):
        """Sets the orpa of this CategoryPolicy.

        If this field (Override Reserve Price Allowed) is returned as <code>true</code>, all leaf categories on the site allow the seller to completely remove the reserve price for an active auction listing. For information on removing or reducing reserve prices, see <a href=\"https://www.ebay.com/help/selling/listings/selling-auctions/reserve-prices?id=4143\"  target=\"_blank\">Setting a reserve price</a>.  # noqa: E501

        :param orpa: The orpa of this CategoryPolicy.  # noqa: E501
        :type: bool
        """

        self._orpa = orpa

    @property
    def orra(self):
        """Gets the orra of this CategoryPolicy.  # noqa: E501

        If this field (Override Reduce Reserve Allowed) is returned as <code>true</code>, the seller can reduce or remove a reserve price that had already been reduced for an active auction listing.  # noqa: E501

        :return: The orra of this CategoryPolicy.  # noqa: E501
        :rtype: bool
        """
        return self._orra

    @orra.setter
    def orra(self, orra):
        """Sets the orra of this CategoryPolicy.

        If this field (Override Reduce Reserve Allowed) is returned as <code>true</code>, the seller can reduce or remove a reserve price that had already been reduced for an active auction listing.  # noqa: E501

        :param orra: The orra of this CategoryPolicy.  # noqa: E501
        :type: bool
        """

        self._orra = orra

    @property
    def payment_methods(self):
        """Gets the payment_methods of this CategoryPolicy.  # noqa: E501

        An array that indicates the acceptable offline payment methods that can be used when listing an item for sale in the corresponding category.  # noqa: E501

        :return: The payment_methods of this CategoryPolicy.  # noqa: E501
        :rtype: list[str]
        """
        return self._payment_methods

    @payment_methods.setter
    def payment_methods(self, payment_methods):
        """Sets the payment_methods of this CategoryPolicy.

        An array that indicates the acceptable offline payment methods that can be used when listing an item for sale in the corresponding category.  # noqa: E501

        :param payment_methods: The payment_methods of this CategoryPolicy.  # noqa: E501
        :type: list[str]
        """

        self._payment_methods = payment_methods

    @property
    def reduce_reserve_allowed(self):
        """Gets the reduce_reserve_allowed of this CategoryPolicy.  # noqa: E501

        If this field (Reduce Reserve Allowed) is <code>true</code>, the corresponding leaf category allows the seller to reduce an item's reserve price. If false, this field is not returned in the response and the corresponding leaf category on the site do not normally allow sellers to reduce an item's reserve price.<p>This field is only returned when <code>true</code> (not returned when false).</p>  # noqa: E501

        :return: The reduce_reserve_allowed of this CategoryPolicy.  # noqa: E501
        :rtype: bool
        """
        return self._reduce_reserve_allowed

    @reduce_reserve_allowed.setter
    def reduce_reserve_allowed(self, reduce_reserve_allowed):
        """Sets the reduce_reserve_allowed of this CategoryPolicy.

        If this field (Reduce Reserve Allowed) is <code>true</code>, the corresponding leaf category allows the seller to reduce an item's reserve price. If false, this field is not returned in the response and the corresponding leaf category on the site do not normally allow sellers to reduce an item's reserve price.<p>This field is only returned when <code>true</code> (not returned when false).</p>  # noqa: E501

        :param reduce_reserve_allowed: The reduce_reserve_allowed of this CategoryPolicy.  # noqa: E501
        :type: bool
        """

        self._reduce_reserve_allowed = reduce_reserve_allowed

    @property
    def upc_support(self):
        """Gets the upc_support of this CategoryPolicy.  # noqa: E501

        This enumerated value indicates whether or not the category on the specified eBay site supports the use of Universal Product Codes (UPCs) to help create a listing. For implementation help, refer to <a href='https://developer.ebay.com/api-docs/sell/metadata/types/sel:ProductIdentiferEnabledEnum'>eBay API documentation</a>  # noqa: E501

        :return: The upc_support of this CategoryPolicy.  # noqa: E501
        :rtype: str
        """
        return self._upc_support

    @upc_support.setter
    def upc_support(self, upc_support):
        """Sets the upc_support of this CategoryPolicy.

        This enumerated value indicates whether or not the category on the specified eBay site supports the use of Universal Product Codes (UPCs) to help create a listing. For implementation help, refer to <a href='https://developer.ebay.com/api-docs/sell/metadata/types/sel:ProductIdentiferEnabledEnum'>eBay API documentation</a>  # noqa: E501

        :param upc_support: The upc_support of this CategoryPolicy.  # noqa: E501
        :type: str
        """

        self._upc_support = upc_support

    @property
    def virtual(self):
        """Gets the virtual of this CategoryPolicy.  # noqa: E501

        If this field is returned as <code>true</code>, the corresponding category is an eBay virtual category, a category in which items may not be listed.<p>This field is only returned when <code>true</code> (not returned when false).</p>  # noqa: E501

        :return: The virtual of this CategoryPolicy.  # noqa: E501
        :rtype: bool
        """
        return self._virtual

    @virtual.setter
    def virtual(self, virtual):
        """Sets the virtual of this CategoryPolicy.

        If this field is returned as <code>true</code>, the corresponding category is an eBay virtual category, a category in which items may not be listed.<p>This field is only returned when <code>true</code> (not returned when false).</p>  # noqa: E501

        :param virtual: The virtual of this CategoryPolicy.  # noqa: E501
        :type: bool
        """

        self._virtual = virtual

    def to_dict(self):
        """Returns the model properties as a dict"""
        result = {}

        for attr, _ in six.iteritems(self.swagger_types):
            value = getattr(self, attr)
            if isinstance(value, list):
                result[attr] = list(map(
                    lambda x: x.to_dict() if hasattr(x, "to_dict") else x,
                    value
                ))
            elif hasattr(value, "to_dict"):
                result[attr] = value.to_dict()
            elif isinstance(value, dict):
                result[attr] = dict(map(
                    lambda item: (item[0], item[1].to_dict())
                    if hasattr(item[1], "to_dict") else item,
                    value.items()
                ))
            else:
                result[attr] = value
        if issubclass(CategoryPolicy, dict):
            for key, value in self.items():
                result[key] = value

        return result

    def to_str(self):
        """Returns the string representation of the model"""
        return pprint.pformat(self.to_dict())

    def __repr__(self):
        """For `print` and `pprint`"""
        return self.to_str()

    def __eq__(self, other):
        """Returns true if both objects are equal"""
        if not isinstance(other, CategoryPolicy):
            return False

        return self.__dict__ == other.__dict__

    def __ne__(self, other):
        """Returns true if both objects are not equal"""
        return not self == other



# coding: utf-8

"""
    EDIS public shipping API

    <div class=\"msgbox_important\"><p class=\"msgbox_importantInDiv\" data-mc-autonum=\"&lt;b&gt;&lt;span style=&quot;color: #dd1e31;&quot; class=&quot;mcFormatColor&quot;&gt;Important! &lt;/span&gt;&lt;/b&gt;\"><span class=\"autonumber\"><span><b><span style=\"color: #dd1e31;\" class=\"mcFormatColor\">Important!</span></b></span></span> This method is only available for Greater-China based sellers with an active eDIS account.</p></div><br>This API allows 3rd party developers in the Greater-China area to process package shipping details.  # noqa: E501

    OpenAPI spec version: 1.0.0
    
    Generated by: https://github.com/swagger-api/swagger-codegen.git
"""

import pprint
import re  # noqa: F401

import six

class ItemRequest(object):
    """NOTE: This class is auto generated by the swagger code generator program.

    Do not edit the class manually.
    """
    """
    Attributes:
      swagger_types (dict): The key is attribute name
                            and the value is attribute type.
      attribute_map (dict): The key is attribute name
                            and the value is json key in definition.
    """
    swagger_types = {
        'buyer_id': 'str',
        'buyer_tax_id': 'str',
        'buyer_tax_type': 'str',
        'ebay_collect_and_remit_tax': 'bool',
        'ebay_collect_and_remit_taxes_value': 'str',
        'email': 'str',
        'item_title': 'str',
        'listing_id': 'str',
        'message': 'str',
        'order_id': 'str',
        'order_line_item': 'str',
        'payment_date': 'str',
        'pay_pal_email': 'str',
        'pay_pal_message': 'str',
        'posted_quantity': 'int',
        'site_id': 'int',
        'sku': 'SkuRequest',
        'sold_date': 'str',
        'sold_price': 'Amount',
        'sold_quantity': 'int',
        'transaction_id': 'str'
    }

    attribute_map = {
        'buyer_id': 'buyerId',
        'buyer_tax_id': 'buyerTaxId',
        'buyer_tax_type': 'buyerTaxType',
        'ebay_collect_and_remit_tax': 'ebayCollectAndRemitTax',
        'ebay_collect_and_remit_taxes_value': 'ebayCollectAndRemitTaxesValue',
        'email': 'email',
        'item_title': 'itemTitle',
        'listing_id': 'listingId',
        'message': 'message',
        'order_id': 'orderId',
        'order_line_item': 'orderLineItem',
        'payment_date': 'paymentDate',
        'pay_pal_email': 'payPalEmail',
        'pay_pal_message': 'payPalMessage',
        'posted_quantity': 'postedQuantity',
        'site_id': 'siteId',
        'sku': 'sku',
        'sold_date': 'soldDate',
        'sold_price': 'soldPrice',
        'sold_quantity': 'soldQuantity',
        'transaction_id': 'transactionId'
    }

    def __init__(self, buyer_id=None, buyer_tax_id=None, buyer_tax_type=None, ebay_collect_and_remit_tax=None, ebay_collect_and_remit_taxes_value=None, email=None, item_title=None, listing_id=None, message=None, order_id=None, order_line_item=None, payment_date=None, pay_pal_email=None, pay_pal_message=None, posted_quantity=None, site_id=None, sku=None, sold_date=None, sold_price=None, sold_quantity=None, transaction_id=None):  # noqa: E501
        """ItemRequest - a model defined in Swagger"""  # noqa: E501
        self._buyer_id = None
        self._buyer_tax_id = None
        self._buyer_tax_type = None
        self._ebay_collect_and_remit_tax = None
        self._ebay_collect_and_remit_taxes_value = None
        self._email = None
        self._item_title = None
        self._listing_id = None
        self._message = None
        self._order_id = None
        self._order_line_item = None
        self._payment_date = None
        self._pay_pal_email = None
        self._pay_pal_message = None
        self._posted_quantity = None
        self._site_id = None
        self._sku = None
        self._sold_date = None
        self._sold_price = None
        self._sold_quantity = None
        self._transaction_id = None
        self.discriminator = None
        if buyer_id is not None:
            self.buyer_id = buyer_id
        if buyer_tax_id is not None:
            self.buyer_tax_id = buyer_tax_id
        if buyer_tax_type is not None:
            self.buyer_tax_type = buyer_tax_type
        if ebay_collect_and_remit_tax is not None:
            self.ebay_collect_and_remit_tax = ebay_collect_and_remit_tax
        if ebay_collect_and_remit_taxes_value is not None:
            self.ebay_collect_and_remit_taxes_value = ebay_collect_and_remit_taxes_value
        if email is not None:
            self.email = email
        if item_title is not None:
            self.item_title = item_title
        if listing_id is not None:
            self.listing_id = listing_id
        if message is not None:
            self.message = message
        if order_id is not None:
            self.order_id = order_id
        if order_line_item is not None:
            self.order_line_item = order_line_item
        if payment_date is not None:
            self.payment_date = payment_date
        if pay_pal_email is not None:
            self.pay_pal_email = pay_pal_email
        if pay_pal_message is not None:
            self.pay_pal_message = pay_pal_message
        if posted_quantity is not None:
            self.posted_quantity = posted_quantity
        if site_id is not None:
            self.site_id = site_id
        if sku is not None:
            self.sku = sku
        if sold_date is not None:
            self.sold_date = sold_date
        if sold_price is not None:
            self.sold_price = sold_price
        if sold_quantity is not None:
            self.sold_quantity = sold_quantity
        if transaction_id is not None:
            self.transaction_id = transaction_id

    @property
    def buyer_id(self):
        """Gets the buyer_id of this ItemRequest.  # noqa: E501

        The buyer's eBay user ID.  # noqa: E501

        :return: The buyer_id of this ItemRequest.  # noqa: E501
        :rtype: str
        """
        return self._buyer_id

    @buyer_id.setter
    def buyer_id(self, buyer_id):
        """Sets the buyer_id of this ItemRequest.

        The buyer's eBay user ID.  # noqa: E501

        :param buyer_id: The buyer_id of this ItemRequest.  # noqa: E501
        :type: str
        """

        self._buyer_id = buyer_id

    @property
    def buyer_tax_id(self):
        """Gets the buyer_tax_id of this ItemRequest.  # noqa: E501

        The tax ID for the buyer. The type of Tax ID is specified in the <b>buyerTaxIdType</b> field.  # noqa: E501

        :return: The buyer_tax_id of this ItemRequest.  # noqa: E501
        :rtype: str
        """
        return self._buyer_tax_id

    @buyer_tax_id.setter
    def buyer_tax_id(self, buyer_tax_id):
        """Sets the buyer_tax_id of this ItemRequest.

        The tax ID for the buyer. The type of Tax ID is specified in the <b>buyerTaxIdType</b> field.  # noqa: E501

        :param buyer_tax_id: The buyer_tax_id of this ItemRequest.  # noqa: E501
        :type: str
        """

        self._buyer_tax_id = buyer_tax_id

    @property
    def buyer_tax_type(self):
        """Gets the buyer_tax_type of this ItemRequest.  # noqa: E501

        The type of tax ID that was supplied by the buyer during the checkout process.  # noqa: E501

        :return: The buyer_tax_type of this ItemRequest.  # noqa: E501
        :rtype: str
        """
        return self._buyer_tax_type

    @buyer_tax_type.setter
    def buyer_tax_type(self, buyer_tax_type):
        """Sets the buyer_tax_type of this ItemRequest.

        The type of tax ID that was supplied by the buyer during the checkout process.  # noqa: E501

        :param buyer_tax_type: The buyer_tax_type of this ItemRequest.  # noqa: E501
        :type: str
        """

        self._buyer_tax_type = buyer_tax_type

    @property
    def ebay_collect_and_remit_tax(self):
        """Gets the ebay_collect_and_remit_tax of this ItemRequest.  # noqa: E501

        This boolean indicates if the corresponding line item is subject to a 'Collect and Remit' tax that eBay will collect and remit to the proper taxing authority on the buyer's behalf.  # noqa: E501

        :return: The ebay_collect_and_remit_tax of this ItemRequest.  # noqa: E501
        :rtype: bool
        """
        return self._ebay_collect_and_remit_tax

    @ebay_collect_and_remit_tax.setter
    def ebay_collect_and_remit_tax(self, ebay_collect_and_remit_tax):
        """Sets the ebay_collect_and_remit_tax of this ItemRequest.

        This boolean indicates if the corresponding line item is subject to a 'Collect and Remit' tax that eBay will collect and remit to the proper taxing authority on the buyer's behalf.  # noqa: E501

        :param ebay_collect_and_remit_tax: The ebay_collect_and_remit_tax of this ItemRequest.  # noqa: E501
        :type: bool
        """

        self._ebay_collect_and_remit_tax = ebay_collect_and_remit_tax

    @property
    def ebay_collect_and_remit_taxes_value(self):
        """Gets the ebay_collect_and_remit_taxes_value of this ItemRequest.  # noqa: E501

        The value of the 'Collect and Remit' tax for the order that the line item belongs to.  # noqa: E501

        :return: The ebay_collect_and_remit_taxes_value of this ItemRequest.  # noqa: E501
        :rtype: str
        """
        return self._ebay_collect_and_remit_taxes_value

    @ebay_collect_and_remit_taxes_value.setter
    def ebay_collect_and_remit_taxes_value(self, ebay_collect_and_remit_taxes_value):
        """Sets the ebay_collect_and_remit_taxes_value of this ItemRequest.

        The value of the 'Collect and Remit' tax for the order that the line item belongs to.  # noqa: E501

        :param ebay_collect_and_remit_taxes_value: The ebay_collect_and_remit_taxes_value of this ItemRequest.  # noqa: E501
        :type: str
        """

        self._ebay_collect_and_remit_taxes_value = ebay_collect_and_remit_taxes_value

    @property
    def email(self):
        """Gets the email of this ItemRequest.  # noqa: E501

        The buyer's email address.  # noqa: E501

        :return: The email of this ItemRequest.  # noqa: E501
        :rtype: str
        """
        return self._email

    @email.setter
    def email(self, email):
        """Sets the email of this ItemRequest.

        The buyer's email address.  # noqa: E501

        :param email: The email of this ItemRequest.  # noqa: E501
        :type: str
        """

        self._email = email

    @property
    def item_title(self):
        """Gets the item_title of this ItemRequest.  # noqa: E501

        The title of the listing.  # noqa: E501

        :return: The item_title of this ItemRequest.  # noqa: E501
        :rtype: str
        """
        return self._item_title

    @item_title.setter
    def item_title(self, item_title):
        """Sets the item_title of this ItemRequest.

        The title of the listing.  # noqa: E501

        :param item_title: The item_title of this ItemRequest.  # noqa: E501
        :type: str
        """

        self._item_title = item_title

    @property
    def listing_id(self):
        """Gets the listing_id of this ItemRequest.  # noqa: E501

        The unique identifier of the eBay listing.<br><br><span class=\"tablenote\"><b>Note:</b> Either a <b>listingId</b> and <b>transactionId</b> pair, or an <b>orderLineItem</b> field is required to identify an order line item.</span>  # noqa: E501

        :return: The listing_id of this ItemRequest.  # noqa: E501
        :rtype: str
        """
        return self._listing_id

    @listing_id.setter
    def listing_id(self, listing_id):
        """Sets the listing_id of this ItemRequest.

        The unique identifier of the eBay listing.<br><br><span class=\"tablenote\"><b>Note:</b> Either a <b>listingId</b> and <b>transactionId</b> pair, or an <b>orderLineItem</b> field is required to identify an order line item.</span>  # noqa: E501

        :param listing_id: The listing_id of this ItemRequest.  # noqa: E501
        :type: str
        """

        self._listing_id = listing_id

    @property
    def message(self):
        """Gets the message of this ItemRequest.  # noqa: E501

        A message from the seller to the buyer.  # noqa: E501

        :return: The message of this ItemRequest.  # noqa: E501
        :rtype: str
        """
        return self._message

    @message.setter
    def message(self, message):
        """Sets the message of this ItemRequest.

        A message from the seller to the buyer.  # noqa: E501

        :param message: The message of this ItemRequest.  # noqa: E501
        :type: str
        """

        self._message = message

    @property
    def order_id(self):
        """Gets the order_id of this ItemRequest.  # noqa: E501

        The unique identifier of the eBay order that the line item belongs to.  # noqa: E501

        :return: The order_id of this ItemRequest.  # noqa: E501
        :rtype: str
        """
        return self._order_id

    @order_id.setter
    def order_id(self, order_id):
        """Sets the order_id of this ItemRequest.

        The unique identifier of the eBay order that the line item belongs to.  # noqa: E501

        :param order_id: The order_id of this ItemRequest.  # noqa: E501
        :type: str
        """

        self._order_id = order_id

    @property
    def order_line_item(self):
        """Gets the order_line_item of this ItemRequest.  # noqa: E501

        The unique identifier of the line item.<br><br><span class=\"tablenote\"><b>Note:</b> Either a <b>listingId</b> and <b>transactionId</b> pair, or an <b>orderLineItem</b> field is required to identify an order line item.<span>  # noqa: E501

        :return: The order_line_item of this ItemRequest.  # noqa: E501
        :rtype: str
        """
        return self._order_line_item

    @order_line_item.setter
    def order_line_item(self, order_line_item):
        """Sets the order_line_item of this ItemRequest.

        The unique identifier of the line item.<br><br><span class=\"tablenote\"><b>Note:</b> Either a <b>listingId</b> and <b>transactionId</b> pair, or an <b>orderLineItem</b> field is required to identify an order line item.<span>  # noqa: E501

        :param order_line_item: The order_line_item of this ItemRequest.  # noqa: E501
        :type: str
        """

        self._order_line_item = order_line_item

    @property
    def payment_date(self):
        """Gets the payment_date of this ItemRequest.  # noqa: E501

        The date and time the buyer pays for the order.<br><br><b>Format:</b> <code>YYYY-MM-DDTHH:MM:SS.SSSZ</code>  # noqa: E501

        :return: The payment_date of this ItemRequest.  # noqa: E501
        :rtype: str
        """
        return self._payment_date

    @payment_date.setter
    def payment_date(self, payment_date):
        """Sets the payment_date of this ItemRequest.

        The date and time the buyer pays for the order.<br><br><b>Format:</b> <code>YYYY-MM-DDTHH:MM:SS.SSSZ</code>  # noqa: E501

        :param payment_date: The payment_date of this ItemRequest.  # noqa: E501
        :type: str
        """

        self._payment_date = payment_date

    @property
    def pay_pal_email(self):
        """Gets the pay_pal_email of this ItemRequest.  # noqa: E501

        The buyer's PayPal email address.  # noqa: E501

        :return: The pay_pal_email of this ItemRequest.  # noqa: E501
        :rtype: str
        """
        return self._pay_pal_email

    @pay_pal_email.setter
    def pay_pal_email(self, pay_pal_email):
        """Sets the pay_pal_email of this ItemRequest.

        The buyer's PayPal email address.  # noqa: E501

        :param pay_pal_email: The pay_pal_email of this ItemRequest.  # noqa: E501
        :type: str
        """

        self._pay_pal_email = pay_pal_email

    @property
    def pay_pal_message(self):
        """Gets the pay_pal_message of this ItemRequest.  # noqa: E501

        The buyer's PayPal message.  # noqa: E501

        :return: The pay_pal_message of this ItemRequest.  # noqa: E501
        :rtype: str
        """
        return self._pay_pal_message

    @pay_pal_message.setter
    def pay_pal_message(self, pay_pal_message):
        """Sets the pay_pal_message of this ItemRequest.

        The buyer's PayPal message.  # noqa: E501

        :param pay_pal_message: The pay_pal_message of this ItemRequest.  # noqa: E501
        :type: str
        """

        self._pay_pal_message = pay_pal_message

    @property
    def posted_quantity(self):
        """Gets the posted_quantity of this ItemRequest.  # noqa: E501

        The quantity of the line item that will be shipped with the package. This value can differ from the order quantity in the case of a combined or split shipment.<br><br><b>Min</b>: 1<br><br><b>Max</b>: 999  # noqa: E501

        :return: The posted_quantity of this ItemRequest.  # noqa: E501
        :rtype: int
        """
        return self._posted_quantity

    @posted_quantity.setter
    def posted_quantity(self, posted_quantity):
        """Sets the posted_quantity of this ItemRequest.

        The quantity of the line item that will be shipped with the package. This value can differ from the order quantity in the case of a combined or split shipment.<br><br><b>Min</b>: 1<br><br><b>Max</b>: 999  # noqa: E501

        :param posted_quantity: The posted_quantity of this ItemRequest.  # noqa: E501
        :type: int
        """

        self._posted_quantity = posted_quantity

    @property
    def site_id(self):
        """Gets the site_id of this ItemRequest.  # noqa: E501

        The unique identifier of the eBay site in which the order occurred.  # noqa: E501

        :return: The site_id of this ItemRequest.  # noqa: E501
        :rtype: int
        """
        return self._site_id

    @site_id.setter
    def site_id(self, site_id):
        """Sets the site_id of this ItemRequest.

        The unique identifier of the eBay site in which the order occurred.  # noqa: E501

        :param site_id: The site_id of this ItemRequest.  # noqa: E501
        :type: int
        """

        self._site_id = site_id

    @property
    def sku(self):
        """Gets the sku of this ItemRequest.  # noqa: E501


        :return: The sku of this ItemRequest.  # noqa: E501
        :rtype: SkuRequest
        """
        return self._sku

    @sku.setter
    def sku(self, sku):
        """Sets the sku of this ItemRequest.


        :param sku: The sku of this ItemRequest.  # noqa: E501
        :type: SkuRequest
        """

        self._sku = sku

    @property
    def sold_date(self):
        """Gets the sold_date of this ItemRequest.  # noqa: E501

        The date the item was sold.<br><br><b>Format</b>: <code>YYYY-MM-DDTHH:MM:SS.SSSZ</code>  # noqa: E501

        :return: The sold_date of this ItemRequest.  # noqa: E501
        :rtype: str
        """
        return self._sold_date

    @sold_date.setter
    def sold_date(self, sold_date):
        """Sets the sold_date of this ItemRequest.

        The date the item was sold.<br><br><b>Format</b>: <code>YYYY-MM-DDTHH:MM:SS.SSSZ</code>  # noqa: E501

        :param sold_date: The sold_date of this ItemRequest.  # noqa: E501
        :type: str
        """

        self._sold_date = sold_date

    @property
    def sold_price(self):
        """Gets the sold_price of this ItemRequest.  # noqa: E501


        :return: The sold_price of this ItemRequest.  # noqa: E501
        :rtype: Amount
        """
        return self._sold_price

    @sold_price.setter
    def sold_price(self, sold_price):
        """Sets the sold_price of this ItemRequest.


        :param sold_price: The sold_price of this ItemRequest.  # noqa: E501
        :type: Amount
        """

        self._sold_price = sold_price

    @property
    def sold_quantity(self):
        """Gets the sold_quantity of this ItemRequest.  # noqa: E501

        The quantity of the line item that was sold.  # noqa: E501

        :return: The sold_quantity of this ItemRequest.  # noqa: E501
        :rtype: int
        """
        return self._sold_quantity

    @sold_quantity.setter
    def sold_quantity(self, sold_quantity):
        """Sets the sold_quantity of this ItemRequest.

        The quantity of the line item that was sold.  # noqa: E501

        :param sold_quantity: The sold_quantity of this ItemRequest.  # noqa: E501
        :type: int
        """

        self._sold_quantity = sold_quantity

    @property
    def transaction_id(self):
        """Gets the transaction_id of this ItemRequest.  # noqa: E501

        The unique identifier of the monetary transaction.<br><br><span class=\"tablenote\"><b>Note:</b> Either a <b>listingId</b> and <b>transactionId</b> pair, or an <b>orderLineItem</b> field is required to identify an order line item.</span>  # noqa: E501

        :return: The transaction_id of this ItemRequest.  # noqa: E501
        :rtype: str
        """
        return self._transaction_id

    @transaction_id.setter
    def transaction_id(self, transaction_id):
        """Sets the transaction_id of this ItemRequest.

        The unique identifier of the monetary transaction.<br><br><span class=\"tablenote\"><b>Note:</b> Either a <b>listingId</b> and <b>transactionId</b> pair, or an <b>orderLineItem</b> field is required to identify an order line item.</span>  # noqa: E501

        :param transaction_id: The transaction_id of this ItemRequest.  # noqa: E501
        :type: str
        """

        self._transaction_id = transaction_id

    def to_dict(self):
        """Returns the model properties as a dict"""
        result = {}

        for attr, _ in six.iteritems(self.swagger_types):
            value = getattr(self, attr)
            if isinstance(value, list):
                result[attr] = list(map(
                    lambda x: x.to_dict() if hasattr(x, "to_dict") else x,
                    value
                ))
            elif hasattr(value, "to_dict"):
                result[attr] = value.to_dict()
            elif isinstance(value, dict):
                result[attr] = dict(map(
                    lambda item: (item[0], item[1].to_dict())
                    if hasattr(item[1], "to_dict") else item,
                    value.items()
                ))
            else:
                result[attr] = value
        if issubclass(ItemRequest, dict):
            for key, value in self.items():
                result[key] = value

        return result

    def to_str(self):
        """Returns the string representation of the model"""
        return pprint.pformat(self.to_dict())

    def __repr__(self):
        """For `print` and `pprint`"""
        return self.to_str()

    def __eq__(self, other):
        """Returns true if both objects are equal"""
        if not isinstance(other, ItemRequest):
            return False

        return self.__dict__ == other.__dict__

    def __ne__(self, other):
        """Returns true if both objects are not equal"""
        return not self == other



# coding: utf-8

"""
    Notification API

    The eBay Notification API enables management of the entire end-to-end eBay notification experience by allowing users to:<ul><li>Browse for supported notification topics and retrieve topic details</li><li>Create, configure, and manage notification destination endpoints</li><li>Configure, manage, and test notification subscriptions</li><li>Process eBay notifications and verify the integrity of the message payload</li></ul>  # noqa: E501

    OpenAPI spec version: v1.6.1
    
    Generated by: https://github.com/swagger-api/swagger-codegen.git
"""

from __future__ import absolute_import

import re  # noqa: F401

# python 2 and python 3 compatibility library
import six

from ...commerce_notification.api_client import ApiClient


class PublicKeyApi(object):
    """NOTE: This class is auto generated by the swagger code generator program.

    Do not edit the class manually.
    Ref: https://github.com/swagger-api/swagger-codegen
    """

    def __init__(self, api_client=None):
        if api_client is None:
            api_client = ApiClient()
        self.api_client = api_client

    def get_public_key(self, public_key_id, **kwargs):  # noqa: E501
        """get_public_key  # noqa: E501

        This method allows users to retrieve a public key using a specified key ID. The public key that is returned in the response payload is used to process and validate eBay notifications.<br><br>The public key ID, which is a required request parameter for this method, is retrieved from the Base64-encoded <b>X-EBAY-SIGNATURE</b> header that is included in the eBay notification.<br><br><div class=\"msgbox_important\"><p class=\"msgbox_importantInDiv\" data-mc-autonum=\"&lt;b&gt;&lt;span style=&quot;color: #dd1e31;&quot; class=&quot;mcFormatColor&quot;&gt;Important! &lt;/span&gt;&lt;/b&gt;\"><span class=\"autonumber\"><span><b><span style=\"color: #dd1e31;\" class=\"mcFormatColor\">Important!</span></b></span></span> The retrieved public key value should be cached for a temporary — but reasonable — amount of time (e.g., one-hour is recommended.) This key should not be requested for every notification since doing so can result in exceeding <a href=\"/develop/apis/api-call-limits\" target=\"_blank\">API call limits</a> if a large number of notification requests is received.</p></div><br><span class=\"tablenote\"><b>Note:</b> For more details about how to process eBay push notifications and validate notification message payloads, see the <a href=\"/api-docs/commerce/notification/overview.html\">Notification API overview</a>.</span>  # noqa: E501
        This method makes a synchronous HTTP request by default. To make an
        asynchronous HTTP request, please pass async_req=True
        >>> thread = api.get_public_key(public_key_id, async_req=True)
        >>> result = thread.get()

        :param async_req bool
        :param str public_key_id: The unique key ID that is used to retrieve the public key.<br><br><span class=\"tablenote\"><b>Note: </b>This is retrieved from the <b>X-EBAY-SIGNATURE</b> header that is included with the push notification.</span> (required)
        :return: PublicKey
                 If the method is called asynchronously,
                 returns the request thread.
        """
        kwargs['_return_http_data_only'] = True
        if kwargs.get('async_req'):
            return self.get_public_key_with_http_info(public_key_id, **kwargs)  # noqa: E501
        else:
            (data) = self.get_public_key_with_http_info(public_key_id, **kwargs)  # noqa: E501
            return data

    def get_public_key_with_http_info(self, public_key_id, **kwargs):  # noqa: E501
        """get_public_key  # noqa: E501

        This method allows users to retrieve a public key using a specified key ID. The public key that is returned in the response payload is used to process and validate eBay notifications.<br><br>The public key ID, which is a required request parameter for this method, is retrieved from the Base64-encoded <b>X-EBAY-SIGNATURE</b> header that is included in the eBay notification.<br><br><div class=\"msgbox_important\"><p class=\"msgbox_importantInDiv\" data-mc-autonum=\"&lt;b&gt;&lt;span style=&quot;color: #dd1e31;&quot; class=&quot;mcFormatColor&quot;&gt;Important! &lt;/span&gt;&lt;/b&gt;\"><span class=\"autonumber\"><span><b><span style=\"color: #dd1e31;\" class=\"mcFormatColor\">Important!</span></b></span></span> The retrieved public key value should be cached for a temporary — but reasonable — amount of time (e.g., one-hour is recommended.) This key should not be requested for every notification since doing so can result in exceeding <a href=\"/develop/apis/api-call-limits\" target=\"_blank\">API call limits</a> if a large number of notification requests is received.</p></div><br><span class=\"tablenote\"><b>Note:</b> For more details about how to process eBay push notifications and validate notification message payloads, see the <a href=\"/api-docs/commerce/notification/overview.html\">Notification API overview</a>.</span>  # noqa: E501
        This method makes a synchronous HTTP request by default. To make an
        asynchronous HTTP request, please pass async_req=True
        >>> thread = api.get_public_key_with_http_info(public_key_id, async_req=True)
        >>> result = thread.get()

        :param async_req bool
        :param str public_key_id: The unique key ID that is used to retrieve the public key.<br><br><span class=\"tablenote\"><b>Note: </b>This is retrieved from the <b>X-EBAY-SIGNATURE</b> header that is included with the push notification.</span> (required)
        :return: PublicKey
                 If the method is called asynchronously,
                 returns the request thread.
        """

        all_params = ['public_key_id']  # noqa: E501
        all_params.append('async_req')
        all_params.append('_return_http_data_only')
        all_params.append('_preload_content')
        all_params.append('_request_timeout')

        params = locals()
        for key, val in six.iteritems(params['kwargs']):
            if key not in all_params:
                raise TypeError(
                    "Got an unexpected keyword argument '%s'"
                    " to method get_public_key" % key
                )
            params[key] = val
        del params['kwargs']
        # verify the required parameter 'public_key_id' is set
        if ('public_key_id' not in params or
                params['public_key_id'] is None):
            raise ValueError("Missing the required parameter `public_key_id` when calling `get_public_key`")  # noqa: E501

        collection_formats = {}

        path_params = {}
        if 'public_key_id' in params:
            path_params['public_key_id'] = params['public_key_id']  # noqa: E501

        query_params = []

        header_params = {}

        form_params = []
        local_var_files = {}

        body_params = None
        # HTTP header `Accept`
        header_params['Accept'] = self.api_client.select_header_accept(
            ['application/json'])  # noqa: E501

        # Authentication setting
        auth_settings = ['api_auth']  # noqa: E501

        return self.api_client.call_api(
            '/public_key/{public_key_id}', 'GET',
            path_params,
            query_params,
            header_params,
            body=body_params,
            post_params=form_params,
            files=local_var_files,
            response_type='PublicKey',  # noqa: E501
            auth_settings=auth_settings,
            async_req=params.get('async_req'),
            _return_http_data_only=params.get('_return_http_data_only'),
            _preload_content=params.get('_preload_content', True),
            _request_timeout=params.get('_request_timeout'),
            collection_formats=collection_formats)



# coding: utf-8

"""
    EDIS public shipping API

    <div class=\"msgbox_important\"><p class=\"msgbox_importantInDiv\" data-mc-autonum=\"&lt;b&gt;&lt;span style=&quot;color: #dd1e31;&quot; class=&quot;mcFormatColor&quot;&gt;Important! &lt;/span&gt;&lt;/b&gt;\"><span class=\"autonumber\"><span><b><span style=\"color: #dd1e31;\" class=\"mcFormatColor\">Important!</span></b></span></span> This method is only available for Greater-China based sellers with an active eDIS account.</p></div><br>This API allows 3rd party developers in the Greater-China area to process package shipping details.  # noqa: E501

    OpenAPI spec version: 1.0.0
    
    Generated by: https://github.com/swagger-api/swagger-codegen.git
"""

import pprint
import re  # noqa: F401

import six

class GetBatteryQualListResponsesData(object):
    """NOTE: This class is auto generated by the swagger code generator program.

    Do not edit the class manually.
    """
    """
    Attributes:
      swagger_types (dict): The key is attribute name
                            and the value is attribute type.
      attribute_map (dict): The key is attribute name
                            and the value is json key in definition.
    """
    swagger_types = {
        'href': 'str',
        'limit': 'int',
        'next': 'str',
        'offset': 'int',
        'prev': 'str',
        'qualifications': 'list[BatteryQualResponses]',
        'total': 'int'
    }

    attribute_map = {
        'href': 'href',
        'limit': 'limit',
        'next': 'next',
        'offset': 'offset',
        'prev': 'prev',
        'qualifications': 'qualifications',
        'total': 'total'
    }

    def __init__(self, href=None, limit=None, next=None, offset=None, prev=None, qualifications=None, total=None):  # noqa: E501
        """GetBatteryQualListResponsesData - a model defined in Swagger"""  # noqa: E501
        self._href = None
        self._limit = None
        self._next = None
        self._offset = None
        self._prev = None
        self._qualifications = None
        self._total = None
        self.discriminator = None
        if href is not None:
            self.href = href
        if limit is not None:
            self.limit = limit
        if next is not None:
            self.next = next
        if offset is not None:
            self.offset = offset
        if prev is not None:
            self.prev = prev
        if qualifications is not None:
            self.qualifications = qualifications
        if total is not None:
            self.total = total

    @property
    def href(self):
        """Gets the href of this GetBatteryQualListResponsesData.  # noqa: E501

        The URL to the current page of the result set.  # noqa: E501

        :return: The href of this GetBatteryQualListResponsesData.  # noqa: E501
        :rtype: str
        """
        return self._href

    @href.setter
    def href(self, href):
        """Sets the href of this GetBatteryQualListResponsesData.

        The URL to the current page of the result set.  # noqa: E501

        :param href: The href of this GetBatteryQualListResponsesData.  # noqa: E501
        :type: str
        """

        self._href = href

    @property
    def limit(self):
        """Gets the limit of this GetBatteryQualListResponsesData.  # noqa: E501

        The value of the <b>limit</b> parameter submitted in the request.  # noqa: E501

        :return: The limit of this GetBatteryQualListResponsesData.  # noqa: E501
        :rtype: int
        """
        return self._limit

    @limit.setter
    def limit(self, limit):
        """Sets the limit of this GetBatteryQualListResponsesData.

        The value of the <b>limit</b> parameter submitted in the request.  # noqa: E501

        :param limit: The limit of this GetBatteryQualListResponsesData.  # noqa: E501
        :type: int
        """

        self._limit = limit

    @property
    def next(self):
        """Gets the next of this GetBatteryQualListResponsesData.  # noqa: E501

        The URI for the next page of results. This value is returned if there is an additional page of results to return from the result set.  # noqa: E501

        :return: The next of this GetBatteryQualListResponsesData.  # noqa: E501
        :rtype: str
        """
        return self._next

    @next.setter
    def next(self, next):
        """Sets the next of this GetBatteryQualListResponsesData.

        The URI for the next page of results. This value is returned if there is an additional page of results to return from the result set.  # noqa: E501

        :param next: The next of this GetBatteryQualListResponsesData.  # noqa: E501
        :type: str
        """

        self._next = next

    @property
    def offset(self):
        """Gets the offset of this GetBatteryQualListResponsesData.  # noqa: E501

        The value of the <b>offset</b> parameter submitted in the request.  # noqa: E501

        :return: The offset of this GetBatteryQualListResponsesData.  # noqa: E501
        :rtype: int
        """
        return self._offset

    @offset.setter
    def offset(self, offset):
        """Sets the offset of this GetBatteryQualListResponsesData.

        The value of the <b>offset</b> parameter submitted in the request.  # noqa: E501

        :param offset: The offset of this GetBatteryQualListResponsesData.  # noqa: E501
        :type: int
        """

        self._offset = offset

    @property
    def prev(self):
        """Gets the prev of this GetBatteryQualListResponsesData.  # noqa: E501

        The URI for the previous page of results. This is returned if there is a previous page of results from the result set.  # noqa: E501

        :return: The prev of this GetBatteryQualListResponsesData.  # noqa: E501
        :rtype: str
        """
        return self._prev

    @prev.setter
    def prev(self, prev):
        """Sets the prev of this GetBatteryQualListResponsesData.

        The URI for the previous page of results. This is returned if there is a previous page of results from the result set.  # noqa: E501

        :param prev: The prev of this GetBatteryQualListResponsesData.  # noqa: E501
        :type: str
        """

        self._prev = prev

    @property
    def qualifications(self):
        """Gets the qualifications of this GetBatteryQualListResponsesData.  # noqa: E501

        This array returns information about each battery qualification associated with an eDIS account, such as the battery type, qualification name, and expiration date.  # noqa: E501

        :return: The qualifications of this GetBatteryQualListResponsesData.  # noqa: E501
        :rtype: list[BatteryQualResponses]
        """
        return self._qualifications

    @qualifications.setter
    def qualifications(self, qualifications):
        """Sets the qualifications of this GetBatteryQualListResponsesData.

        This array returns information about each battery qualification associated with an eDIS account, such as the battery type, qualification name, and expiration date.  # noqa: E501

        :param qualifications: The qualifications of this GetBatteryQualListResponsesData.  # noqa: E501
        :type: list[BatteryQualResponses]
        """

        self._qualifications = qualifications

    @property
    def total(self):
        """Gets the total of this GetBatteryQualListResponsesData.  # noqa: E501

        The total number of results that match the input criteria.  # noqa: E501

        :return: The total of this GetBatteryQualListResponsesData.  # noqa: E501
        :rtype: int
        """
        return self._total

    @total.setter
    def total(self, total):
        """Sets the total of this GetBatteryQualListResponsesData.

        The total number of results that match the input criteria.  # noqa: E501

        :param total: The total of this GetBatteryQualListResponsesData.  # noqa: E501
        :type: int
        """

        self._total = total

    def to_dict(self):
        """Returns the model properties as a dict"""
        result = {}

        for attr, _ in six.iteritems(self.swagger_types):
            value = getattr(self, attr)
            if isinstance(value, list):
                result[attr] = list(map(
                    lambda x: x.to_dict() if hasattr(x, "to_dict") else x,
                    value
                ))
            elif hasattr(value, "to_dict"):
                result[attr] = value.to_dict()
            elif isinstance(value, dict):
                result[attr] = dict(map(
                    lambda item: (item[0], item[1].to_dict())
                    if hasattr(item[1], "to_dict") else item,
                    value.items()
                ))
            else:
                result[attr] = value
        if issubclass(GetBatteryQualListResponsesData, dict):
            for key, value in self.items():
                result[key] = value

        return result

    def to_str(self):
        """Returns the string representation of the model"""
        return pprint.pformat(self.to_dict())

    def __repr__(self):
        """For `print` and `pprint`"""
        return self.to_str()

    def __eq__(self, other):
        """Returns true if both objects are equal"""
        if not isinstance(other, GetBatteryQualListResponsesData):
            return False

        return self.__dict__ == other.__dict__

    def __ne__(self, other):
        """Returns true if both objects are not equal"""
        return not self == other
